/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary for library manipulation.
 */
var Library = function(){
	var config = {
		currentBookId: '',
		open: false,
		libraryView: '',
		XUL_NS: "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
		XHTML_NS: "http://www.w3.org/1999/xhtml",
		bookMetadata: []
	}
	
    var treeView = function(){
        this.rowCount = config.bookMetadata.length;
        this.getCellText = function(row, col){
        	if(col.id === 'created_date') {
        		var d = new Date(+config.bookMetadata[row][col.id]);
        		return d.toLocaleDateString();
        	} 
        	return  config.bookMetadata[row][col.id];
        };
        this.getCellValue = function(row, col){ return  config.bookMetadata[row][col.id]; };
        this.setTree = function(treebox){ this.treebox = treebox; };
        this.isEditable = function(row, col){ return col.editable; };
        this.isContainer = function(row){ return false; };
        this.isSeparator = function(row){ return false; };
        this.isSorted = function(){ return false; };
        this.getLevel = function(row){ return 0; };
        this.getImageSrc = function(row, col){ return null; };
        this.getRowProperties = function(row, props){};
        this.getCellProperties = function(row, col, props){};
        this.getColumnProperties = function(colid, col, props){};
        this.cycleHeader = function(col, elem){};
    };
	
	/**
	 * Remove all the books from the variable
	 */
	var clear = function(){
		config.bookMetadata.splice(0, config.bookMetadata.length);
	}
	
	/**
	 * Initilize the library.
	 */
	var init = function(){
		var self = this;
		
		//Remove all the books from library which is used to display the library.
		self.clear();
		//Just the following variable will make the library to populate.
		config.bookMetadata = BookMetadata.getAll();
		//set the shelf description at the top of library.
		self.showShelfDescription('All Books', 'Contains all purchased or uploaded books.');
		//Sort the books in the library
		self.sort();
		window.resizeTo(UserPref.config.openWidth, UserPref.config.openHeight);
	}
	
	/**
	 * Sets the library view. table|thumbnail.
	 */
	var setLibraryView = function(){
		if(config.libraryView == 'table') {
			document.getElementById('library-azd').view = new treeView();
		}
		else if(config.libraryView == 'thumbnail'){
			$('#show_thumbnail_view').html('');
			
			if(config.bookMetadata.length == 0) {
				//If library is empty, then display message "No books available"
				var emptyDiv = document.createElementNS(config.XHTML_NS, "html:div");
				emptyDiv.setAttribute('class', 'no-thumbnail');
				var emptyP = document.createElementNS(config.XHTML_NS, "html:p");
				var emptyText = document.createTextNode('No books available');
				emptyP.appendChild(emptyText);
				emptyDiv.appendChild(emptyP);
				document.getElementById('show_thumbnail_view').appendChild(emptyDiv);
			} else{
				//Show the books in their own container, with buttons and cover image
				for (var i = 0; i < config.bookMetadata.length; i++) {
					var bookId = config.bookMetadata[i]['id'];
					var bookTitle = config.bookMetadata[i]['title'];
					var bookAuthor = config.bookMetadata[i]['author'];
					var bookCover = config.bookMetadata[i]['cover_image'];
					var cover_type = config.bookMetadata[i]['cover_type'];
					
					var container = document.createElementNS(config.XHTML_NS, "html:div");
					if(cover_type == 'image') {
						container.setAttribute('class', 'thumbnail-item-image');
					} else if(cover_type == 'avatar') {
						container.setAttribute('class', 'thumbnail-item-avatar');
					}
					
					container.style.backgroundImage = "url("+bookCover+")";
					container.setAttribute('ondragstart', 'event.dataTransfer.setData("text/plain", "'+bookId+'")');
					container.setAttribute('tooltiptext', bookTitle);
					container.setAttribute('onclick', 'javascript:AME.openBook("'+bookId+'")');
					
					var details = document.createElementNS(config.XHTML_NS, "html:div");
					details.setAttribute('class', 'thumb-details');
					
					var bookName = document.createElementNS(config.XHTML_NS, "html:p");
					var text1 = document.createTextNode(bookTitle);
					bookName.appendChild(text1);
					details.appendChild(bookName);
					
					var author = document.createElementNS(config.XHTML_NS, "html:p");
					var text2 = document.createTextNode(bookAuthor);
					author.appendChild(text2);
					details.appendChild(author);
					
					container.appendChild(details);
					
					var btn_container = document.createElementNS(config.XHTML_NS, "html:div");
					btn_container.setAttribute('class', 'thumb-buttons');
					
					var btn1 = document.createElementNS(config.XHTML_NS, "html:button");
					btn1.setAttribute('type', 'button');
					btn1.setAttribute('class', 'thumbnail-delete-book');
					btn1.setAttribute('style', 'width: 40px; height: 24px;');
					btn1.setAttribute('onclick', 'javascript: AME.deleteBook("'+bookId+'");event.stopPropagation();');
					var btnImg = document.createElementNS(config.XHTML_NS, 'html:img');
					btnImg.setAttribute('src', 'icons/ico_delete-book_18.png')
					btn1.appendChild(btnImg);
					btn_container.appendChild(btn1);
					
					var btn2 = document.createElementNS(config.XHTML_NS, "html:button");
					btn2.setAttribute('type', 'button');
					btn1.setAttribute('class', 'thumbnail-book-info');
					btn2.setAttribute('style', 'width: 40px; height: 24px;');
					btn2.setAttribute('onclick', 'javascript: AME.showBookInfoDialog("'+bookId+'");event.stopPropagation();');
					var btnImg = document.createElementNS(config.XHTML_NS, 'html:img');
					btnImg.setAttribute('src', 'icons/ico_book-info_18.png')
					btn2.appendChild(btnImg);
					btn_container.appendChild(btn2);
					
					container.appendChild(btn_container);
					
					document.getElementById('show_thumbnail_view').appendChild(container);
				}
			}
		}
		else if(config.libraryView == 'stack'){
			//	TODO: Implement library view in stack mode.
		}
	}
	
	/**
	 * Sorts the library depending upon the column clicked.
	 * @params column Name of the column to sort against
	 */
	var sort = function(column){
		var self = this;
		
		var tree = document.getElementById("library-azd");
		var order = tree.getAttribute("sortDirection") == "ascending" ? 1 : -1;
		var columnName;
		if (column) {
			columnName = column.id;
			if (tree.getAttribute("sortResource") == columnName) {
				order *= -1;
			}
		}
		else {
			//If column is not provided then sort against created_date.
			columnName = tree.getAttribute("sortResource");
		}
		
		var columnSort = function(a, b){
			if(self.prepareForComparison(a[columnName]) > self.prepareForComparison(b[columnName])) 
	            return 1 * order;
	        if(self.prepareForComparison(a[columnName]) < self.prepareForComparison(b[columnName])) 
	            return -1 * order;
	        //tie breaker: name ascending is the second level sort
	        if(columnName != "title"){
	            if (self.prepareForComparison(a["title"]) > self.prepareForComparison(b["title"])) 
	                return 1;
	            if (self.prepareForComparison(a["title"]) < self.prepareForComparison(b["title"])) 
	                return -1;
	        }
	        return 0;
	    }
	
		config.bookMetadata.sort(columnSort);
		
		tree.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
		tree.setAttribute("sortResource", columnName);
	};
	
	/**
	 * Converts the string to lowercase.
	 * @param o String to convert
	 */
	var prepareForComparison = function(o){
		if(typeof o == "string"){
			return o.toLowerCase();
		}
		return o;
	};
	
	/**
	 * This function returns the bookId from the clicked row.
	 * @returns {string} returns the bookId.
	 */
	var getClickedBook = function(){
		var tree = document.getElementById("library-azd");
    	var cellIndex = 0;
	    var cellText = tree.view.getCellText(tree.currentIndex, tree.columns.getColumnAt(cellIndex));
	    return cellText;
	};
	
	/**
	 * Deletes the book from library
	 * @params bookId Id of the book to be deleted
	 */
	var deleteBook = function(bookId){
		var self = this;
		
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'library_master';
		var condition = new Array(['id', bookId, 'TEXT']);
		deleteRowFromDatabase(fileObj, table_name, condition);
		
		table_name = 'shelf_document_join';
		var condition = new Array(['lib_id', bookId, 'INTEGER']);
		deleteRowFromDatabase(fileObj, table_name, condition);
	};
	
	/**
	 * Displays books only from particular shelf
	 * @params shelfId Id of the shelf
	 * @params shelfName Name of the shelf
	 * @params shelfDescription Short description of the shelf
	 */
	var showShelfBooks = function(shelfId, shelfName, shelfDescription){
		var self = this;
		
		//Remove all the books from library which is used to display the library.
		self.clear();
		//Just the following variable will make the library to populate.
		config.bookMetadata = BookMetadata.getShelfBooks(shelfId);
		//Sort the books in the library
		self.sort();
		//Set the library view, table|thumbnail
		self.setLibraryView();
		//set the shelf description at the top of library.
		self.showShelfDescription(shelfName, shelfDescription);
	};
	
	/**
	 * Sets the shelf name and description at the top of library.
	 * @params shelfName Name of the shelf
	 * @params shelfDescription Description of the shelf.
	 */
	var showShelfDescription = function(shelfName, shelfDescription){
		//Shorten the name and description if length greater than 80 chars.
		var shelfNameShort = shelfName,
			shelfDescriptionShort = shelfDescription;
		if (shelfName.length > 80) {
			shelfNameShort = shelfName.substr(0, 77) + '...';
		}
		if (shelfDescription.length > 80) {
			shelfDescriptionShort = shelfDescription.substr(0, 77) + '...';
		}
		$('.shelf-name-ape').val(shelfNameShort.toUpperCase());
		$('.shelf-name-ape').attr('tooltiptext', shelfName.toUpperCase());
		$('.shelf-description-ape').val(shelfDescriptionShort);
		$('.shelf-description-ape').attr('tooltiptext', shelfDescription);
	};
	
	return {
		config: config,
		treeView: treeView,
		clear: clear,
		init: init,
		sort: sort,
		prepareForComparison: prepareForComparison,
		setLibraryView: setLibraryView,
		getClickedBook: getClickedBook,
		deleteBook: deleteBook,
		showShelfBooks: showShelfBooks,
		showShelfDescription: showShelfDescription
	};
}();