/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains functions necessary for shelf operations.
 */
var Shelf = function(){
    var config = {
    	open: false,
    	prompts: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),
		XUL_NS: "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
		currentShelfId: '1',
		currentShelfName: 'All Books',
		currentShelfDescription: 'Contains all purchased or uploaded books.'
	}
	
	var init = function(){
		var self = this;
		
		self.clear();
		var shelves = self.get();
		shelves = self.sort(shelves);
		self.generate(shelves);
		if(config.currentShelfId != '') {
			$('#'+config.currentShelfId).addClass('shelf-clicked');
		} else {
			$('#1').addClass('shelf-clicked');
		}
    }
	
	var generate = function(shelves){
		var self = this;
		
		for(i = 0; i < shelves.length; i++){
			var shelfId = shelves[i][0],
			    shelfType = shelves[i][1],
			    shelfName = escape(shelves[i][2]),
			    shelfDescription = escape(shelves[i][3]);
			
			var shelfItem = document.createElementNS(self.config.XUL_NS, "description");
			shelfItem.setAttribute("id", shelfId);
			shelfItem.setAttribute("value", unescape(shelfName));
			shelfItem.setAttribute("context", "shelf_context_menu");
			shelfItem.setAttribute("tooltiptext", unescape(shelfName)+' :\n'+unescape(shelfDescription));
			shelfItem.setAttribute("onclick", "Shelf.showBooks(id, '"+shelfName+"', '"+shelfDescription+"')");
			
			if(shelfName != 'All Books'){
				shelfItem.setAttribute("ondragenter", "Shelf.bookDragEnter(event)");
				shelfItem.setAttribute("ondragover", "Shelf.bookDragOver(event)");
				shelfItem.setAttribute("ondrop", "AME.addBookToShelf(event, '"+shelfName+"')");
			}
			
			document.getElementById('leftcol-aza').appendChild(shelfItem);
		}
	}
	
	var bookDragEnter = function(event){
		event.stopPropagation();
		event.preventDefault();
	}
	
	var bookDragOver = function(event){
		event.stopPropagation();
		event.preventDefault();
	}
	
	var clear = function(){
		$("#leftcol-aza > description").remove();
	}
	
	/**
	 * Function to get the list of available shelves.
	 * @returns {array} returns an array of shelves.
	 */
	var get = function(){
		var self = this;
		
		//Check if the database file exists.
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()){
			return [];
		}
		
		//Get the information of all the shelves from the database and return.
		var values = ['id', 'shelf_type', 'shelf_name', 'shelf_description'];
		var table_name = 'shelf_master';
		var statement = getValuesFromDatabase(fileObj, values, table_name);
		if(!statement){
			return [];
		}
		
		var shelves = new Array();
		while (statement.executeStep()) {
			var data = new Array();
			data.push(statement.row.id);
			data.push(statement.row.shelf_type);
			data.push(statement.row.shelf_name);
			data.push(statement.row.shelf_description);
			shelves.push(data);
		}
		statement.finalize();
		
		return shelves;
	}
	
	var sort = function(shelves) {
		var self = this;
		
		var unsortableShelves = new Array();
		for(var i = 0; i < shelves.length; i++) {
			if(shelves[i][1] == 'default') {
				var a = shelves.splice(i, 1);
				unsortableShelves.push(a[0]);
			}
		}
		 
		var columnSort = function(a, b) {
			if (a[2].toLowerCase() < b[2].toLowerCase()) return -1;
			if (a[2].toLowerCase() > b[2].toLowerCase()) return 1;
			return 0;
		}
		
		shelves.sort(columnSort);
		
		shelves = unsortableShelves.concat(shelves);
		
		return shelves;
	}
	
	/**
	 * Function to create a shelf.
	 * @param type Type of the shelf.
	 * @param name Name of the shelf.
	 * @param description A short description of the shelf.
	 * @returns {boolean} true if shelf created, false otherwise.
	 */
	var create = function(type, name, description) {
		var self = this;
		
		//Get the list of all shelves from the database.
		var shelves = self.get();
		
		//Check if the shelf is already created.
		for (i = 0; i < shelves.length; i++) {
			if (shelves[i][2] == name) {
				return false;
			}
		}
		
		//Check if the database file exists.
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()){
			return false;
		}
		
		//Add new shelf to the database.
		var table_name = 'shelf_master';
		var values = new Array(['shelf_type', type, 'TEXT'],
							   ['shelf_name', name, 'TEXT'],
							   ['shelf_description', description, 'TEXT']);
		insertRowIntoDatabase(fileObj, values, table_name);
		
		return true;
	}
	
	var addBook = function(bookMetadata, shelfName){
		var self = this;
		
		var shelves = self.get();
		var shelfId = '',
		bookId = '';
		for(i = 0; i< shelves.length; i++){
			if(shelves[i][2] == shelfName){
				shelfId = shelves[i][0];
				break;
			}
		}
		if(typeof bookMetadata.id === 'undefined') {
			bookId = BookMetadata.getBookId(bookMetadata.title);
		} else {
			bookId = bookMetadata.id;
		}
		// var allBooks = BookMetadata.getAll();
		// for(i = 0; i< allBooks.length; i++){
			// if(allBooks[i]['title'] == bookMetadata.title){
				// bookId = allBooks[i]['id'];
			// }
		// }
		
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'shelf_document_join';
		var values = new Array(['lib_id', bookId, 'INTEGER'],
		   					  ['shelf_id', shelfId, 'INTEGER']);	
		insertRowIntoDatabase(fileObj, values, table_name);
	}
	
	/**
	 * Delete the book from shelf
	 * @params bookId Id of the book
	 * @params shelfId Id of the shelf
	 */
	var deleteBook = function(bookId, shelfId) {
		var self = this;
		
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'shelf_document_join';
		var condition = new Array(['lib_id', bookId, 'INTEGER'],
								  ['shelf_id', shelfId, 'INTEGER']);
		deleteRowFromDatabase(fileObj, table_name, condition);
	};
	
	var getClicked = function(){
		var id = $('.shelf-clicked').attr('id');
		var label = $('.shelf-clicked').attr('value');
		
		return {'id': id, 'label': label}
	}
	
	var deleteShelf = function(clickedShelf){
		if(clickedShelf.label == 'All Books'){
			config.prompts.alert(null, AME.getTitle(), AzdMessages.cannotDeleteShelf);
			// alert('Can not delete this Shelf !');
			return;
		}
		if(config.prompts.confirm(null, AME.getTitle(), AzdMessages.deleteShelf)){
			var fileObj = getFileObject(AME.getDbFilePath());
		    if (!fileObj.exists()){
		        return false;
		    }
		    var table_name = 'shelf_master';
			var condition = new Array(['id', clickedShelf.id, 'INTEGER']);
			
			// remove shelf from shelf_master table.		
		    deleteRowFromDatabase(fileObj, table_name, condition);
		
			// remove shelf from shelf_document_join
			table_name = 'shelf_document_join';
			condition = new Array(['shelf_id', clickedShelf.id, 'INTEGER']);
			deleteRowFromDatabase(fileObj, table_name, condition);
		}
	}
	
	/**
	 * Highlights the shelf in shelf panel
	 * @params shelfId Id of the shelf
	 */
	var highlight = function(shelfId) {
		$("#leftcol-aza > description").removeClass('shelf-clicked');
		$('#'+shelfId).addClass('shelf-clicked');
	};
	
	/**
	 * Show all books in the shelf
	 * @params shelfId Id of the shelf
	 * @params shelfName Name of the shelf
	 * @params shelfDescription Short description of the shelf 
	 */
	var showBooks = function(shelfId, shelfName, shelfDescription) {
		var self = this;
		
		shelfName = unescape(shelfName);
		shelfDescription = unescape(shelfDescription);
		//Highlight shelf in shelf panel
		self.highlight(shelfId);
		
		config.currentShelfId = shelfId;
		config.currentShelfName = shelfName;
		config.currentShelfDescription = shelfDescription;
		
		//Show the shelf books.
		Library.showShelfBooks(shelfId, shelfName, shelfDescription)
	};
	
    return {
		config: config,
		init: init,
		create: create,
		get: get,
		sort: sort,
		addBook: addBook,
		init: init,
		clear: clear,
		generate: generate,
		getClicked: getClicked,
		deleteShelf: deleteShelf,
		showBooks: showBooks,
		highlight: highlight,
		bookDragEnter: bookDragEnter,
		bookDragOver: bookDragOver,
		deleteBook: deleteBook
    };
}();