/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * AZARDI MAIN ENGINE
 * This is the main js file.
 */
var AME = function(){

    /**
     * Private properties and methods.
     */
   
    var config = {
    	appName: 'AZARDI',
        context: null,
		defaultDirPath: ['Home', 'Infogrid Pacific'],
		dbFilePath: ['Home', 'Infogrid Pacific'],
		cdpDbFilePath: ['Home', 'Infogrid Pacific'],
		searchDbFilePath: ['Home', 'Infogrid Pacific'],
		tempDownloadPath: ['ProfD'],
		title: '',
		splashScreen: '',
		use_catalog: false,
		catalog_id: '',
		register_reader: false,
		register_url: '',
		db_version: '',
		readerToken: null,
		wasShelfOpen: false,
		XUL_NS: 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul',
		prompts: Components.classes['@mozilla.org/embedcomp/prompt-service;1'].getService(Components.interfaces.nsIPromptService),
		isCtrlPressed: false,
		isShiftPressed: false
    };
	
    /**
     * Checks for the updates and downloads update file if available.
     */
    var checkUpdates = function() {
		var updateListener = {
			onProgress: function(request, position, totalSize) {
	                // alert('update check progress');
	        },
            onCheckComplete: function(request, updates, updateCount) {
	            //After retrieving updates.xml successfully, application will execute this code.
	
				if(updateCount > 0) {
	                //Don't ask user for applying updates. Do it at all.
	                var updater = Components.classes["@mozilla.org/updates/update-service;1"].createInstance(Components.interfaces.nsIApplicationUpdateService);
	
	                var update = updater.selectUpdate(updates, updateCount);
	                var download = updater.downloadUpdate(update, false);
	            }
            },
            onError: function(request, update) {
                // alert('update check error');
            },
            QueryInterface: function(aIID) {
                if (! aIID.equals(Components.interfaces.nsIUpdateCheckListener) && ! aIID.equals(Components.interfaces.nsISupports))
                        throw Components.results.NS_ERROR_NO_INTERFACE;
                return this;
            }
		};
	
        var checker = Components.classes["@mozilla.org/updates/update-checker;1"].createInstance(Components.interfaces.nsIUpdateChecker);

        checker.checkForUpdates(updateListener, true);
	}
	
	/**
	 * This is the first function to be called when the application is started.
	 */
    var init = function(){
		var self = this;
		
		//Check if the updates are available.
		//Uncomment the following line before going live.
		//It should be commented while developing the code.
		self.checkUpdates();
		
		/*	
		 	If the application is not updating then to debug, comment above line and uncomment following six lines.
			They provide a decent ui for update procedure and also show errors if any.
		*/
		
		/*
		var um = Components.classes["@mozilla.org/updates/update-manager;1"].getService(Components.interfaces.nsIUpdateManager);
		var prompter = Components.classes["@mozilla.org/updates/update-prompt;1"].createInstance(Components.interfaces.nsIUpdatePrompt);
		if (um.activeUpdate && um.activeUpdate.state == "pending")
			prompter.showUpdateDownloaded(um.activeUpdate);
		else
			prompter.checkForUpdates();
		*/
		
		//Initilize the bootup preferences.
		self.setBootupPreferences();
		
		document.title = config.title;
		
		//Display the splash screen.
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e) {
			$('#doc-viewer-azd').contents().find('#splash_image').attr('src', config.splashScreen);
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		$('#doc-viewer-azd').attr('src', 'html/welcome_azardi2.html');
		
		//Create directories and copy database tables if don't exist.
		config.defaultDirPath.push(config.title);
		
		config.dbFilePath.push(config.title);
		config.dbFilePath.push('azardi.azd');
		
		config.cdpDbFilePath.push(config.title);
		config.cdpDbFilePath.push('cdp_master.azd');
		
		config.searchDbFilePath.push(config.title);
		config.searchDbFilePath.push('azd_search.azd');
		
		createDirectory(config.defaultDirPath);
		self.copyDefaultDbTables();
		
		//Create default shelf, if not already created.
		Shelf.create('default', 'All Books', 'Contains all purchased or uploaded books.');
		
		//Initilize fonts.
		Fonts.init();
		
		//Initilize User Preferences with the values of database.
		UserPref.init();
		
		//Intilize Book Preferences with the values of user preferences.
		BookPref.init();
		
		//Check if you want to show catalog or just library.
		if (config.use_catalog) {
			//Show the splash screen for some time and then load the catalog.
			setTimeout(function() {
				if(config.register_reader) {
					if(!self.checkToken()) {
						$('#doc-viewer-azd').attr('src', 'html/azardi_register_reader.html');
					} else {
						Catalog.load(config.catalog_id);
					}
				}
			}, 2 * 1000);
		} else {
			//Check the database version.
			var dif = self.checkDbVersion();
			if(dif) {
				//Initilize the database updation.
				Update_DB.init();
			}
			//Initilize the toolbar.
			Toolbar.init();
			//Initilize Adding multiple epubs.
			AddMulEpub.init();
			//Initilize the keyboard events.
			self.setupKeyEvent();
		}
		
		window.resizeTo(UserPref.config.openWidth, UserPref.config.openHeight);
		
		//Finally show the main frame.
		$('#doc-viewer-azd').show();
    };
    
    /**
     * This function copies the default empty database tables to users home directory.
     */
    var copyDefaultDbTables = function () {
    	var self = this;
    	
		var searchDbObj = getFileObject(config.searchDbFilePath);
		var cdpDbObj = getFileObject(config.cdpDbFilePath);
		var azaDbObj = getFileObject(config.dbFilePath);
		var destObj = getFileObject(config.defaultDirPath);
		
		if (!searchDbObj.exists()) {
			var sourceObj = getFileObject(['CurProcD', 'chrome', 'content', 'azd_db', 'azd_search.azd']);
			sourceObj.copyTo(destObj, null);
		}
		
		if (!cdpDbObj.exists()) {
			var sourceObj = getFileObject(['CurProcD', 'chrome', 'content', 'azd_db', 'cdp_master.azd']);
			sourceObj.copyTo(destObj, null);
		}
		
		if (!azaDbObj.exists()) {
			var sourceObj = getFileObject(['CurProcD', 'chrome', 'content', 'azd_db', 'azardi.azd']);
			sourceObj.copyTo(destObj, null);
		}
    }
    
    /**
     * Checks if the database version differs.
     * @returns {boolean} true if version differs, false otherwise.
     */
    var checkDbVersion = function () {
		var self = this;
		
		//Get the database version from the database.
		var destinationFile = getFileObject(config.dbFilePath);
    	var table_name = 'System';
		var values = ['value'];
		var conditions = new Array(['key', 'db_version', 'TEXT']);
								   
		var statement = getValuesFromDatabase(destinationFile, values, table_name, conditions);
		var value_d;
		while(statement.executeStep()) {
			value_d = parseFloat(statement.row.value);
		}
		if(typeof value_d == 'undefined'){
			return true;
		}
		//Check the version no. from database with the version no. from azardi-bootup.js.
		if(value_d > parseFloat(config.db_version)){
			return true;
		}
		else{
			return false;
		}
    };
    
    var registerApp = function (orgId){
    	var self = this;
    	
    	var params = {
  						inn:{
  							url: config.register_url,
  							type: 'register_application',
  							catalogId: config.catalog_id,
  							orgId: orgId
  						},
  						out:{
  							out:null,
  							readerToken: null,
  							catDownloadUrl: null
  						}
  					};
  		window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=250, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();
    	
    	if (!params.out.out) {
    		alert("Failed to register the reader.")
  			// alert('Failed to register the reader.\nExiting the application');
  			// window.close();
		}
		else{
			config.readerToken = params.out.readerToken;
			self.downloadCat(params.out.catDownloadUrl+orgId);
			// if(config.catalog_id === 'all') {
				// Catalog.displayAll();
			// } else {
				Catalog.load(config.catalog_id)
			// }
		}
    };
    
    var downloadCat = function(cat_download_url) {
    	var params = {
			inn:{
				cat_download_url: cat_download_url,
				type: 'cat_dai',
			},
			out:{
				out:null,
			}
		};
		window.openDialog("chrome://azardi2/content/azd_modal_dialog.xul", "", "width=270, height=60, chrome, dialog, modal, resizable=yes, titlebar=no, dependent, centerscreen", params).focus();

		if (!params.out.out) {
			alert('Failed to download catalog(s).\nExiting the application');
			window.close();
		}
    }
    
    /**
     * Initilizes the bootup preferences found in azardi-bootup.js.
     */
    var setBootupPreferences = function () {
    	config.title = Bootup.reader_title || 'AZARDI';
    	config.splashScreen = Bootup.reader_splash || '../icons/AZARDI-splash.png';
    	UserPref.config.openWidth = Bootup.width || 500;
    	UserPref.config.openHeight = Bootup.height || 700;
    	config.use_catalog = Bootup.use_catalog;
    	if(Bootup.use_catalog){
    		config.catalog_id = Bootup.catalog_id;
    	}
    	config.register_reader = Bootup.register_reader;
    	config.register_url = Bootup.register_url;
    	config.db_version = Bootup.db_version;
    }
	
	var getDBVersion = function() {
		return config.db_version;
	}
	
	var getDefaultDirPath = function(){
		return config.defaultDirPath;
	}
	
	var getDbFilePath = function(){
		return config.dbFilePath;
	}
	
	var getCdpDbFilePath = function () {
		return config.cdpDbFilePath;
	}
	
	var getSearchDbFilePath = function () {
		return config.searchDbFilePath;
	}
	
	var getTempDownloadPath = function () {
		return config.tempDownloadPath;
	}
	
	// var getCfsLocation = function () {
		// // return config.cfs_location;
	// }
	
	var getCatalogId = function () {
		return config.catalog_id;
	}
	
	var getReaderToken = function () {
		return config.readerToken;
	}
	
	var getAppName = function () {
		return config.appName;
	}
	
	/**
	 * Sets the title of the window to the book title
	 * @params title Title of the book
	 * @params author Author of the book
	 */
	var windowTitle = function(title, author){
		if (title.length > 40) {
	        title = title.substring(0, 37) + '...';
	    }
		if(author.length > 15){
			author = author.substring(0, 12) + '...';
		}
	    document.title = config.title+' :: '+title + ' - ' + author;
	};
	
	/**
	 * Displays the library (table/thumbnail)
	 * @params isBookAdded Is the new book added in the library.
	 */
	var showLibrary = function(isBookAdded) {
		var self = this;
		
		if(Library.config.open || Shelf.config.open){
			return;
		}
		
		if(config.context == 'AZD'){
			Azd.unload();
			Toolbar.closeAzd();
		}
		
		if(config.context == 'ePub'){
			//If the epub was open, save the preferences to database.
			BookPref.save();
			//Unload the epub
			Epub.unload();
			//Hides the epub related buttons from toolbar
			Toolbar.closeEpub();
		}
		
		$('#viewport1-azd').show();
		$('#shelf_label').show();
		$('#doc-viewer-azd').hide();
		
		//	FIXME: Why do we need to show table view here. Not working without this statement. Need to check.
		$('#library-azd').show();
		//Show the buttons related to library.
		Toolbar.showLibrary();
		document.title = config.title;
		
		if(config.wasShelfOpen) {
			//Show the shelf, if shelf was open earlier.
			$('#but_show_shelf').click();
			if(isBookAdded) {
				//Show the books in shelf
				Shelf.showBooks(Shelf.config.currentShelfId, Shelf.config.currentShelfName, Shelf.config.currentShelfDescription);
			}
		} else {
			//Initilize the library
			Library.init();
			config.context  = 'library';
			Library.config.open = true;
			Shelf.config.open = false;
		}
		
		//Set the button status
		if(UserPref.config.libraryView == 'table')
			document.getElementById('but_table').click();
		else if(UserPref.config.libraryView == 'thumbnail')
			document.getElementById('but_thumb').click();
			
		//If new book is added, then scroll to top to show the new books.
		if(isBookAdded) {
			$('#gridbox').scrollTop(0);
			$('#library-azd').scrollTop(0);
		}
	};
	
	/**
	 * Function to hide library related items.
	 */
	var closeLibrary = function(){
		$('#viewport1-azd').hide();
		$('#library-azd').hide();
		$('.gridbox').hide();
		$('#shelf_label').hide();
		Toolbar.closeLibrary();
		Library.config.open = false;
	};
	
	var sortLibrary = function(column){
		Library.sort(column)
		Library.setLibraryView();
	}
	
	/**
	 * Toggles the library view between table and thumbnail.
	 * @params view View to be applied
	 */
	var changeLibraryView = function(view) {
		if(view == 'table'){
			$('#library-azd').show();
			$('.gridbox').hide();
		}
		else if(view == 'thumbnail'){
			$('#library-azd').hide();
			$('.gridbox').show();
		}
		else if(view == 'stack'){
			//	TODO: Implement the stack view.
		}
		Library.config.libraryView = view;
		//Set the library view. table|thumbnail.
		Library.setLibraryView();
	};
	
	var getTitle = function () {
		return config.title;
	}
	
	var showShelf = function(){
		if(Shelf.config.open){
			return;
		}

		var shelfWidth = parseInt(UserPref.config.openWidth) + 250;
		
		window.resizeTo(shelfWidth, UserPref.config.openHeight);
		
		$('#leftcol-aza').show();
		document.title = config.title;
		
		Shelf.init();
		
		config.context = 'shelf';
		// config.wasShelfOpen = true;
		Shelf.config.open = true;
		Library.config.open = false;
	}
	
	/**
	 * Function to hide shelf related elements.
	 */
	var closeShelf = function(){
		if(!Shelf.config.open){
			return;
		}
		
		$('#leftcol-aza').hide();
		
		window.resizeTo(UserPref.config.openWidth, UserPref.config.openHeight);
		
		if(document.getElementById('but_show_shelf').hasAttribute('checked')){
			document.getElementById('but_show_shelf').removeAttribute('checked');
		}
		
		Shelf.config.open = false;
		// config.wasShelfOpen = false;
	};
	
	var toggleShelf = function(){
		var self = this;
		
		if(document.getElementById('but_show_shelf').hasAttribute('checked')){
			self.showShelf();
			config.wasShelfOpen = true;
		}
		else{
			self.closeShelf();
			config.context = 'library';
			Library.config.open = true;
			Shelf.config.open = false;
			config.wasShelfOpen = false;
		}
	}
	
	var resetAddBookForm = function(){
		$('.book-info').val('');
		// $('#cover_thumbnail').attr('src', '');
		$('#cover_thumbnail').css('background-image', '');
	    $('#btn_ok').attr('disabled', true);
	    $('#btn_ok_mul_add').attr('disabled', true);
	    
	    AddMulEpub.clear();
	}
	
	var showAddBookForm = function(){
		var self = this;
		
		self.closeLibrary();
		self.resetAddBookForm();
		self.closeShelf();
		$('#form_add_doc').show();
		$('#file_type').focus();
	}
	
	var closeAddBookForm = function(isBookAdded){
		var self = this;
		
		$('#form_add_doc').hide();
		self.showLibrary(isBookAdded);
	}
	
	/**
	 * Displays the user preferences form
	 */
	var showUserPrefForm = function(){
		var self = this;
		
		if(config.context == 'library')
			self.closeLibrary();
		if (config.context == 'shelf') {
			self.closeLibrary();
			self.closeShelf();
		}
		if (config.context == 'ePub') {
			$('#doc-viewer-azd').hide();
			Toolbar.closeEpub();
		}
		
		UserPref.setForm();
		$('#form_usr_pref').show();
		$('#user_pref_lib_view').focus();
	}
	
	var closeUserPrefForm = function(){
		var self = this;
		
		// self.resizeWindow(UserPref.config.openWidth, UserPref.config.openHeight);
		if(config.context == 'library' || config.context == 'shelf'){
			$('#form_usr_pref').hide();
			self.showLibrary();	
		}
		else if(config.context == 'ePub'){
			$('#form_usr_pref').hide();
			$('#doc-viewer-azd').show();
			Toolbar.showEpub();
			
			self.showLoading();
			
			if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
				Columns.config.showLastPage = false;
				Columns.config.goToDepth = Columns.config.currentReadingDepth;
				Columns.init(BookPref.config.noOfCols);
			}
			else{
				Flow.config.showLastPage = false;
				Flow.init();
			}
			
			self.hideLoading();
		}
	}
	
	var resizeWindow = function(width, height){
		window.resizeTo(width, height)
	}
	
	/**
	 * Intilizes the sequence to save user preferences to database and displays either previously opened book or library
	 */
	var saveUserPrefs = function(){
		var self = this;
		
		UserPref.save();
		if(config.context == 'ePub'){
			BookPref.init();
			BookPref.set();	
			
			Chapter.config.orig_width = UserPref.config.openWidth;
			Chapter.config.orig_height = UserPref.config.openHeight;
			
			if(Chapter.config.azardi_presentation == 'fixed') {
				Chapter.unbindResize();
				
				if(Chapter.config.azardi_width && Chapter.config.azardi_height) {
					if(Chapter.config.azardi_margins != -1) {
						window.innerWidth = parseInt(Chapter.config.azardi_width) + (2 * Chapter.config.azardi_margins);
					} else {
						window.innerWidth = parseInt(Chapter.config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = parseInt(Chapter.config.azardi_height)+parseInt(Chapter.config.toolbar_height);
				} else if(Chapter.config.azardi_width) {
					if(Chapter.config.azardi_margins != -1) {
						window.innerWidth = parseInt(Chapter.config.azardi_width) + (2 * Chapter.config.azardi_margins);
					} else {
						window.innerWidth = parseInt(Chapter.config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = BookPref.config.openHeight;
				} else if(Chapter.config.azardi_height){
					window.innerWidth = BookPref.config.openWidth;
					window.innerHeight = parseInt(Chapter.config.azardi_height)+parseInt(Chapter.config.toolbar_height);
					
				} else {
					window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
				}
			} else if(config.azardi_presentation == 'fullscreen') {
				self.unbindResize();
				width = screen.width || BookPref.config.openWidth;
				height = screen.height || BookPref.config.openHeight;
				window.resizeTo(width, height);
				window.moveTo(0, 0);
			} else if(Chapter.config.azardi_presentation == null) {
				Chapter.unbindResize();
				Chapter.bindResize();
				window.resizeTo(Chapter.config.orig_width, Chapter.config.orig_height);
			}
		} else if(config.context == 'library' || config.context == 'shelf') {
			self.resizeWindow(UserPref.config.openWidth, UserPref.config.openHeight);
		}
		
		self.closeUserPrefForm();
	}
	
	var resetCreateShelfForm = function(){
		$('#shelf_name').val('');
		$('#shelf_description').val('');
		$('#btn_shelf_ok').attr('disabled', 'true');
	};
	
	/**
	 * Displays the add new shelf form.
	 */
	var showCreateShelfForm = function(){
		var self = this;
		
		//Close library and shelf
		self.closeLibrary();
		self.closeShelf();
		//Reset the add new shelf form
		self.resetCreateShelfForm();
		//Finally, show the form
		$('#form_create_shelf').show();
		$('#shelf_name').focus();
	};
	
	var closeCreateShelfForm = function(){
		var self = this;
		
		$('#form_create_shelf').hide();
		self.showLibrary();
	};
	
	var createShelf = function(){
		var self = this;
		
		var type = 'user',
		    name = $('#shelf_name').val(),
		    description = $('#shelf_description').val();
		
		var shelfCreated = Shelf.create(type, name, description);
		
		if(!shelfCreated){
			var message = AzdMessages.dupShelf.replace('%name%', name);
			var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
			prompts.alert(null, AME.getTitle(), message);
			return;
		}
		
		self.closeCreateShelfForm();
	}
	
	var deleteShelf = function(){
		var clickedShelf = Shelf.getClicked();
		Shelf.deleteShelf(clickedShelf);
		Shelf.init();
	}
	
	var addBook = function(){
		var self = this;
		
		var bookMetadata = {};
		bookMetadata.type = $('#file_type').val();
		bookMetadata.uid = $('#book_id').val();
		bookMetadata.created_date = $('#created_date_in_ms').val();
		bookMetadata.title = $('#title').val();
		bookMetadata.author = $('#author').val();
		bookMetadata.publisher_identifier = $('#publisher').val();
		bookMetadata.publish_date = $('#publish_date').val();
		bookMetadata.description = $('#description').val();
		bookMetadata.location = $('#file_location').val();
		bookMetadata.cover_image = $('#cover_image').val();
		bookMetadata.cover_type = $('#cover_type').val();
		bookMetadata.version = $('#version').val();

		BookMetadata.add(bookMetadata);
		
		Shelf.addBook(bookMetadata, 'All Books');
		
		document.getElementById('library-azd').setAttribute("sortResource", "created_date");
		document.getElementById('library-azd').setAttribute("sortDirection", "descending");
		
		self.closeAddBookForm(true);
	};
	
	var addMultipleBooks = function() {
		var self = this;
		
		var allBookMetadata = AddMulEpub.getAllBooksInfo();
		for(var i=0; i<allBookMetadata.length; i++) {
			var bookMetadata = allBookMetadata[i][1];
			BookMetadata.add(bookMetadata);
			Shelf.addBook(bookMetadata, 'All Books');
		}
		
		document.getElementById('library-azd').setAttribute('sortResource', 'created_date');
		document.getElementById('library-azd').setAttribute('sortDirection', 'descending');
		
		self.closeAddBookForm(true);
	};
	
	/**
	 * Initilizes the sequence to completely delete book from database
	 * @params bookId Id of the book to be deleted 
	 */
	var deleteBook = function(bookId){

		var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
		
		if(prompts.confirm(null, AME.getTitle(), AzdMessages.deleteBook)){
			if(typeof bookId == 'undefined') {
				var bookId = Library.getClickedBook();
			}
			
			if(Library.config.open) {
				Library.deleteBook(bookId);
				BookPref.deletePref(bookId);
				Notes.deleteAllNotesFromDatabase(bookId);
			} else if(Shelf.config.open) {
				if(Shelf.config.currentShelfName == 'All Books') {
					Library.deleteBook(bookId);
					BookPref.deletePref(bookId);
					Notes.deleteAllNotesFromDatabase(bookId);
				} else {
					Shelf.deleteBook(bookId, Shelf.config.currentShelfId);
				}
			}
			
			if(config.context == 'library'){
				Library.init();
				Library.setLibraryView();
			}
			else if(config.context == 'shelf'){
				Library.showShelfBooks(Shelf.config.currentShelfId, Shelf.config.currentShelfName, Shelf.config.currentShelfDescription);
			}
		}
	};
	
	var addBookToShelf = function(event, shelfName){
		
		shelfName = unescape(shelfName);
		event.dataTransfer.effectAllowed = "copy";
		var bookId = '';
		if(Library.config.libraryView == 'table'){
			bookId = Library.getClickedBook();
		}
		if(Library.config.libraryView == 'thumbnail'){
			bookId = event.dataTransfer.getData("text/plain");
		}
		var bookMetadata = {};
		bookMetadata['id'] = bookId;
		Shelf.addBook(bookMetadata, shelfName);
	};
	
	/**
	 * Extracts the information from user selected file and fills the add book form
	 */
	var selectFile = function(fileType){
		var self = this;
		
		if (fileType == 'AZD'){
		 	var azdFolder = openSelectFile('Select File', 'directory', 'none');
			//getInfo calls the enterBookInformation from its side as it can not return any value.
			Azd.getInfo(azdFolder);
			//TODO: Do not set image to iframe instead set them directly to the canvas.
	    }
	    else if (fileType == 'ePub'){
			var epubFile = openSelectFile('Select File', 'open', 'ePub');
			
			if(!epubFile) {
				return;
			}
			try{
				var epubInfo = Epub.getInfo(epubFile);
			}
			catch(e){
				//	If any error while loading epubs then display error message and exit.
				var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
				prompts.alert(null, AME.getTitle(), e.message);
				// alert(e.message);
				return;
			}
			self.setCoverThumbnail(epubInfo.coverImagePath);
			self.enterBookInformation(epubInfo.metadata);
			self.changeAddBookOkButtonStatus();
		}
	}
	
	/**
	 * Set the cover thumbnail in add book form
	 * @params coverPath Path of the cover file
	 */
	var setCoverThumbnail = function(coverPath){
		var self = this;
		
		// coverPath = getFileObjectFromPath(coverPath);
		// var coverUrl = getURL(coverPath);
		
		self.getCoverThumbDataUri(coverPath);
		
		
		// if(coverPath.exists()){
			// //	if the cover file is there then only set the url.
			// //	TODO: in earlier functions when i get the coverpath, need to check if the file exists or not and then
			// //	depending on that should fallback on some default cover.
			// $('#pageContainerIframe').attr('src', coverUrl);
		// }
		setTimeout(function(){
			// var coverThumbDataUri = AME.getCoverThumbDataUri();
			// $('#cover_thumbnail').attr('src', coverThumbDataUri);
			// $('#cover_image').val(coverThumbDataUri);
// 			
			// $('#pageContainerIframe').attr('src', '');
			
			//	Here we can delete the extracted folder.
			deleteFolder(getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder]));
		}, 500);
	}
	
	/**
	 * Returns the scaled dimension
	 * @params maxW Maximum width allowed for element
	 * @params maxH Maximum height allowed for element
	 * @params currW Current width of element
	 * @params currH Current height of element
	 */
	var scaleSize = function(maxW, maxH, currW, currH){

		var ratio = currH / currW;
		
		if(currW >= maxW && ratio <= 1){
			currW = maxW;
			currH = currW * ratio;
		} else if(currH >= maxH){
			currH = maxH;
			currW = currH / ratio;
		}
		
		return [currW, currH];
	};
	
	/**
	 * Sets the datauri to the cover container
	 * @params coverPath Path of the cover file
	 */
	var getCoverThumbDataUri = function(coverPath){
		var self = this;
		
		//If the image is svg, do not resize it.
		if(coverPath.split('.').pop() == 'svg') {
			var coverUri = generateDataURI(coverPath);
			$('#cover_thumbnail').css('background-image', 'url('+coverUri+')');
		  	$('#cover_image').val(coverUri);
		} else {
			var coverObject = getFileObjectFromPath(coverPath);
			var coverUrl = getURL(coverObject);
			var canvas = document.getElementById("thumb_canvas");
			var ctx = canvas.getContext("2d");
			var img = new Image();
			img.onload = function(){
				var currW = img.width;
				var currH = img.height;
				var dim = self.scaleSize(300, 300, currW, currH);
				canvas.width = dim[0];
				canvas.height = dim[1];
				ctx.mozImageSmoothingEnabled = true;
				ctx.drawImage(img, 0, 0, dim[0], dim[1]);
				
				var coverUri = canvas.toDataURL("image/png");
			  	$('#cover_thumbnail').css('background-image', 'url('+coverUri+')');
			  	$('#cover_image').val(coverUri);
			};
			img.src = coverUrl;
		}
	};
	
	/**
	 * Enters the book information in add book form
	 */
	var enterBookInformation = function(bookInfo){
		var location = '',
		    bookId = '',
		    title = '',
		    author = '',
		    publisher = '',
		    publishDate = '',
		    description = '',
		    version = '',
		    cover_type = '',
		    currentTime = new Date();
		$.each(bookInfo, function(index, value){
			switch(value[0]){
				case 'dc:location':
					location = value[1];
					break;
				case 'dc:identifier':
					bookId = value[1];
					break;
				case 'dc:title':
					title = value[1];
					break;
				case 'dc:creator':
					author = value[1];
					break;
				case 'dc:publisher':
					publisher = value[1];
					break;
				case 'dc:date':
					publishDate = value[1];
					break;
				case 'dc:description':
					description = value[1];
					break;
				case 'dc:version':
					version = value[1];
					break;
				case 'dc:cover_type':
					cover_type = value[1];
					break;
			}
		});
		
		$('#file_location').val(location);
		$('#book_id').val(bookId);
		$('#version').val(version);
		$('#title').val(title);
		$('#author').val(author);
		$('#publisher').val(publisher);
		$('#created_date').val(currentTime.toLocaleDateString());
		$('#created_date_in_ms').val(currentTime.getTime());
		$('#publish_date').val(publishDate);
		$('#description').val(description);
		$('#cover_type').val(cover_type);
	};
	
	var openBookAtFirst = function() {
		var self  = this;
		
		if(typeof bookId === 'undefined'){
			var bookId = Library.getClickedBook();
		}
		
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if(Library.config.bookMetadata[i]['id'] === +bookId){
				var bookType = Library.config.bookMetadata[i]['type'],
					bookTitle = Library.config.bookMetadata[i]['title']
					bookAuthor = Library.config.bookMetadata[i]['author'],
					bookLocation = Library.config.bookMetadata[i]['location'],
					bookImage = Library.config.bookMetadata[i]['cover_image'],
					bookImageType = Library.config.bookMetadata[i]['cover_type'],
					bookVersion = Library.config.bookMetadata[i]['version'];
				break;
			}
		}
		
		Library.config.currentBookId = bookId;
		
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			self.closeLibrary();
			self.closeShelf();
			$('#doc-viewer-azd').show();
			Toolbar.showEpub();
			Toolbar.disableEpubBtn();
			self.showLoading();
			Epub.load(bookLocation, bookVersion, bookTitle, bookAuthor, bookImageType, true);
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
		
		config.context = 'ePub';
		
		Chapter.history.chapterNo = null;
		Chapter.history.readingDepth = null;
		self.windowTitle(bookTitle, bookAuthor);
	}
	
	/**
	 * This function initiates the sequence to open a book from library.
	 * @params bookId Id of the book to open. 
	 */
	var openBook = function(bookId){
		var self  = this;
		
		//bookId is undefined when it is opened from the table view.
		if(typeof bookId === 'undefined'){
			var bookId = Library.getClickedBook();
		}
		
		// Get the metadata of the clicked book.
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if(Library.config.bookMetadata[i]['id'] === +bookId){
				var bookType = Library.config.bookMetadata[i]['type'],
					bookTitle = Library.config.bookMetadata[i]['title']
					bookAuthor = Library.config.bookMetadata[i]['author'],
					bookLocation = Library.config.bookMetadata[i]['location'],
					bookImage = Library.config.bookMetadata[i]['cover_image'],
					bookImageType = Library.config.bookMetadata[i]['cover_type'],
					bookVersion = Library.config.bookMetadata[i]['version'];
				break;
			}
		}
		
		Library.config.currentBookId = bookId;
		
		//AZD is not used now, see function openBookFromCatalog
		if(bookType == 'AZD'){
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				self.closeLibrary();
				self.closeShelf();
				$('#doc-viewer-azd').show();
				Azd.load(bookLocation, cdpId);
				
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_cdp_doc.html');
			
			Toolbar.showAzd();
			config.context = 'AZD';
		}
		else if(bookType == 'ePub'){
			//	This is required for library to close only when the default epub document is fully loaded.
			
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				// Close the library and the shelf
				self.closeLibrary();
				self.closeShelf();
				$('#doc-viewer-azd').show();
				Toolbar.showEpub();
				Toolbar.disableEpubBtn();
				//show loading overlay
				self.showLoading();
				//Initiate the epub loading sequence.
				Epub.load(bookLocation, bookVersion, bookTitle, bookAuthor, bookImageType, false);
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			
			config.context = 'ePub';
			
			Chapter.history.chapterNo = null;
			Chapter.history.readingDepth = null;
		}
		//set the window title
		self.windowTitle(bookTitle, bookAuthor);
	};
	
	var openBookFromCatalog = function (cdpId) {
		var self = this;
		
		self.hideStatusBar();
		self.removeAllMessages();
		var bookLocation = getFileObject(config.cdpDbFilePath);
		
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			try{
				Azd.load(bookLocation.path, cdpId);
			}catch(e){alert(e);}
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_cdp_doc.html');
		config.context = 'AZD';
	}
	
	/**
	 * Show the loading overlay.
	 */
	var showLoading = function(){
		$('#doc-viewer-azd').contents().find('#loading_overlay').show();
	};
	
	/**
	 * Hide the loading overlay.
	 */
	var hideLoading = function(){
		$('#doc-viewer-azd').contents().find('#loading_overlay').hide();
	};
	
	/**
	 * Hide/show full page toc
	 * @params show Boolean expression.
	 */
	var toggleFullPageToc = function(show){
		if(show){
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e) {
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
				if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
					Chapter.history.readingDepth = Columns.config.currentReadingDepth;
					Chapter.config.oldReadingDepth = Columns.config.currentReadingDepth;
				}
				
				if(Epub.config.version == 3) {
					Epub.setFullPageTocDataEpub3();
				} else {
					Epub.setFullPageTocDataEpub2();
				}
				Notes.getAllNotes(Library.config.currentBookId);
				Notes.sortAllNotes();
				Notes.createAllNotesPage();
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_toc.html');
			Toolbar.showFullPageToc();
		}
		else{
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						Epub.setInPageTocDataEpub3();
					}
					else{
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					Epub.setInPageTocDataEpub2();
				}
				Chapter.goToReadingDepth();
				
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			Toolbar.showEpub();
		}
	}
	
	/**
	 * Show/Hide Landmark page
	 * @params show Boolean expression
	 */
	var toggleLandmark = function (show) {
		var self = this;
		
		if(show) {
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e) {
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
				
				if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
					Chapter.history.readingDepth = Columns.config.currentReadingDepth;
					Chapter.config.oldReadingDepth = Columns.config.currentReadingDepth;
				}
				
				// Set landmark and page-nav data
				Epub.setLandmarkDataEpub3();
				Epub.setPageNavDataEpub3();
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_landmark.html');
			Toolbar.showLandmark();
		} else {
			$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
				if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						Epub.setInPageTocDataEpub3();
					}
					else{
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					Epub.setInPageTocDataEpub2();
				}
				Chapter.goToReadingDepth();
				
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			Toolbar.showEpub();
		}
	};
	
	/**
	 * Initilizes the sequence to go to next page
	 */
	var nextPage = function(){
		Chapter.next();
	};
	
	/**
	 * Initilizes the sequence to go to previous page
	 */
	var previousPage = function(){
		Chapter.previous();
	}
	
	/**
	 * Initilizes the sequence to go to next chapter
	 */
	var nextChapter = function() {
		Chapter.nextChapter();
	};
	
	/**
	 * Initilizes the sequence to go to previous chapter
	 */
	var previousChapter = function() {
		Chapter.previousChapter();
	};
	
	var changeFontSize = function(size){
		var currentSize = parseFloat(Fonts.config.fontSize);
		if(size === 'plus') {
			currentSize += 0.10;
		} else {
			currentSize -= 0.10;
		}
		currentSize = currentSize.toFixed(1);
		if(currentSize <= Fonts.config.maxValue) {
			Fonts.set(currentSize);
			if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
				Columns.initilizeVariables();
				if(Columns.config.noOfCol > 1) {
					Columns.removeEmptyCols();
					Columns.appendEmptyCols(Columns.config.noOfCol);
				}
				Columns.setApproximatePage();
				Columns.getColumnCount();
				Columns.getCurrentColPosition();
				Columns.getCurrentReadingDepth();
				Columns.updateProgressBar();
				Chapter.setNavButtons();
			}
			else{
				Flow.getCurrentReadingDepth();
			}
		}
	}
	/*
	var changeFontSize = function(size){
		Fonts.set(size);
		if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
			Columns.initilizeVariables();
			Columns.getColumnCount();
			Columns.getCurrentColPosition();
			Columns.getCurrentReadingDepth();
			Columns.updateProgressBar();
			Chapter.setNavButtons();
		}
		else{
			Flow.getCurrentReadingDepth();
		}
	}
	*/
	
	var changeLineSpace = function(space){
		var currentSpace = parseFloat(Line.config.lineSpacing);
		if(space === 'plus') {
			currentSpace += 0.10;
		} else {
			currentSpace -= 0.10;
		}
		currentSpace = currentSpace.toFixed(1);
		if(currentSpace >= Line.config.minValue) {
			Line.set(currentSpace);
			if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
				Columns.initilizeVariables();
				if(Columns.config.noOfCol > 1) {
					Columns.removeEmptyCols();
					Columns.appendEmptyCols(Columns.config.noOfCol);
				}
				Columns.setApproximatePage();
				Columns.getColumnCount();
				Columns.getCurrentColPosition();
				Columns.getCurrentReadingDepth();
				Columns.updateProgressBar();
				Chapter.setNavButtons();
			}
			else{
				Flow.getCurrentReadingDepth();
			}
		}
	}
	
	/*
	var changeLineSpace = function(space){
		Line.set(space);
		if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
			Columns.initilizeVariables();
			Columns.getColumnCount();
			Columns.getCurrentColPosition();
			Columns.getCurrentReadingDepth();
			Columns.updateProgressBar();
			Chapter.setNavButtons();
		}
		else{
			Flow.getCurrentReadingDepth();
		}
	}
	*/
	// var changeParagraphSpace = function(space){
		// Paragraph.set(space);
		// if(BookPref.config.columns){
			// Columns.initilizeVariables();
			// Columns.getColumnCount();
			// Columns.getCurrentColPosition();
			// Columns.getCurrentReadingDepth();
			// Columns.updateProgressBar();
		// }
	
	var toggleJustify = function(justify){
		Justify.set(justify);
	}
	
	var toggleHyphen = function(hyphenate){
		Hyphen.set(hyphenate);
	}
	
	var toggleAudioControls = function(display) {
		if(display) {
			Toolbar.showAudioControls();
		} else {
			Toolbar.hideAudioControls();
		}
	};
	
	var changeMarginSpace = function(space){
		if(Chapter.config.azardi_margins == -1 || Chapter.config.azardi_presentation != 'fixed') {
			var currentSpace = parseInt(Margin.config.margin);
			if(space === 'plus') {
				currentSpace += 10;
			} else {
				currentSpace -= 10;
			}
			
			if(currentSpace >= Margin.config.minValue) {
				Margin.set(currentSpace);
				if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
					Columns.initilizeVariables();
					if(Columns.config.noOfCol > 1) {
						Columns.removeEmptyCols();
						Columns.appendEmptyCols(Columns.config.noOfCol);
					}
					Columns.setApproximatePage();
					Columns.getColumnCount();
					Columns.getCurrentColPosition();
					Columns.getCurrentReadingDepth();
					Columns.updateProgressBar();
					Chapter.setNavButtons();
				}
				else{
					Flow.getCurrentReadingDepth();
				}
			}
		}
	}
	
	/*
	var changeMarginSpace = function(space){
		if(Chapter.config.azardi_margins == -1 || Chapter.config.azardi_presentation != 'fixed') {
			Margin.set(space);
			if(BookPref.config.columns && Chapter.config.azardi_presentation != 'fixed'){
				Columns.initilizeVariables();
				Columns.getColumnCount();
				Columns.getCurrentColPosition();
				Columns.getCurrentReadingDepth();
				Columns.updateProgressBar();
				Chapter.setNavButtons();
			}
			else{
				Flow.getCurrentReadingDepth();
			}
		}
	}
	*/
	var changeBackgroundColor = function(color){
		
		var colorCode;
		
		switch(color) {
			case 'Normal':
				colorCode = '#FFFFFF';
				break;
			case 'Warm':
				colorCode = '#FAFAD2';
				break;
			case 'Low Contrast':
				colorCode = '#F0F0F0';
				break;
		}
		
		Color.set(colorCode);
	}
	
	/**
	 * Opens a notes dialog
	 */
	var openNotesDialog = function(){
		var self = this;
		
		//Check if valid to create notes
		var create = Notes.isTextSelected();
		if(create){
			var serial = document.getElementById('doc-viewer-azd').contentWindow.serializeSelection();
			var selectionString = Notes.getSelectedString();
			
			$('#doc-viewer-azd').contents().find('#notes-serialized').text(serial);
			$('#doc-viewer-azd').contents().find('#notes-book-id').text(Library.config.currentBookId);
			$('#doc-viewer-azd').contents().find('#notes-current-chapter').text(Chapter.config.currentChapter);
			$('#doc-viewer-azd').contents().find('#notes-selection-string').text(selectionString);
			
			$('#doc-viewer-azd').contents().find('#white_overlay').show();
			$('#doc-viewer-azd').contents().find('#add_notes').show();
			
			$('#doc-viewer-azd').contents().find('#notesDescription').focus();
			
			$('#book_context_menu').hide();
			
			// self.unbindArrowKeys(document.getElementById('doc-viewer-azd').contentWindow.document);
		}
	};
	
	/**
	 * Closes the notes dialog
	 */
	var closeAddNotesDialog = function(){
		var self = this;
		
		$('#doc-viewer-azd').contents().find('#white_overlay').hide();
		$('#doc-viewer-azd').contents().find('#add_notes').hide();
		
		$('#doc-viewer-azd').contents().find('#notesTitle').val('');
		$('#doc-viewer-azd').contents().find('#notesDescription').val('');
		
		$('#doc-viewer-azd').contents().find('#notes-serialized').html('');
		$('#doc-viewer-azd').contents().find('#notes-book-id').html('');
		$('#doc-viewer-azd').contents().find('#notes-current-chapter').html('');
		$('#doc-viewer-azd').contents().find('#notes-selection-string').html('');
		
		$('#book_context_menu').show();
		
		// self.bindArrowKeys(document.getElementById('doc-viewer-azd').contentWindow.document);
	};
	
	/**
	 * Initilizes the sequence to save notes
	 */
	var saveNotes = function(){
		var self = this;
		
		var notesDescription = $('#doc-viewer-azd').contents().find('#notesDescription').val();
		if(notesDescription == '' || notesDescription == ' ' || notesDescription.replace(/\s/g, '') == ''){
			self.closeAddNotesDialog();
			return;	
		}
		
		var serial = $('#doc-viewer-azd').contents().find('#notes-serialized').text(),
		    selectionString = $('#doc-viewer-azd').contents().find('#notes-selection-string').text(),
		    bookId = $('#doc-viewer-azd').contents().find('#notes-book-id').text(),
		    currentChapter = $('#doc-viewer-azd').contents().find('#notes-current-chapter').text();
//			   notesTitle = $('#doc-viewer-azd').contents().find('#notesTitle').val(),
//			   notesDescription = $('#doc-viewer-azd').contents().find('#notesDescription').val();
		
		var currentTime = new Date().getTime();
		Notes.save(bookId, currentChapter, notesDescription, selectionString, currentTime, serial, '', false);
		var noteId = Notes.getLatestId();
		Notes.addToArray(notesDescription, selectionString, serial, currentTime, noteId);
		Notes.highlightOnTheFly(serial, noteId);
//		document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
		self.closeAddNotesDialog();
	};
	
	var saveEditedNotes = function(){
		var self =this;
		
		var noteId = $('#doc-viewer-azd').contents().find('#id_anno').html();
		var notesDescription = $('#doc-viewer-azd').contents().find('.notes-description-ape').html();
		var currentTime = new Date().getTime();
		
		Notes.save('', '', notesDescription, '', currentTime, '', noteId, true);
		Notes.updateArray(notesDescription, currentTime, noteId);
		self.closeEditNotesDialog();
	}
	
	/**
	 * Opens the notes window in edit mode
	 * @params noteId Id of the note to be edited
	 */
	var openEditNotesDialog = function(noteId){
		var self = this;
		
		Notes.createEditDialog();
		Notes.setEditDialog(noteId);
		
		$('#doc-viewer-azd').contents().find('#white_overlay').show();
		$('#doc-viewer-azd').contents().find('#edit_notes').show();
		
		$('#doc-viewer-azd').contents().find('#edit_notes').css('height', document.getElementById('doc-viewer-azd').contentWindow.config.currentHeight *0.4 + 'px');
		$('#doc-viewer-azd').contents().find('#edit_notes').css('top', document.getElementById('doc-viewer-azd').contentWindow.config.currentHeight *0.3 + 'px');
		
		$(window).resize(function(){
			$('#doc-viewer-azd').contents().find('#edit_notes').css('height', document.getElementById('doc-viewer-azd').contentWindow.config.currentHeight *0.4 + 'px');
			$('#doc-viewer-azd').contents().find('#edit_notes').css('top', document.getElementById('doc-viewer-azd').contentWindow.config.currentHeight *0.3 + 'px');
		});
		
		$('#book_context_menu').hide();
		
		// self.unbindArrowKeys(document.getElementById('doc-viewer-azd').contentWindow.document);
	}
	
	var closeEditNotesDialog = function(){
		var self = this;
		
		$('#doc-viewer-azd').contents().find('#white_overlay').hide();
		$('#doc-viewer-azd').contents().find('#edit_notes').remove();
		
		$('#book_context_menu').show();
		
		// self.bindArrowKeys(document.getElementById('doc-viewer-azd').contentWindow.document);
	}
	
	var changeShelfOkButtonStatus = function(){
		var shelfName = $('#shelf_name').val();
		
		if (shelfName == '' || shelfName == ' ' || shelfName.replace(/ /g, '') == ''){
			$('#btn_shelf_ok').attr('disabled', true);
		}
		else{
			$('#btn_shelf_ok').attr('disabled', false);
		}
	}
	
	/**
	 * Change the status of OK button in add book form
	 */
	var changeAddBookOkButtonStatus = function(){
		var bookName = $('#title').val();
		var bookLocation = $('#file_location').val();
		
		if (bookName == '' || bookName == ' ' || bookName.replace(/ /g, '') == '' || bookLocation == '' || bookLocation == ' '){
			$('#btn_ok').attr('disabled', true);
		}
		else{
			$('#btn_ok').attr('disabled', false);
		}
	};
	
	var changeMulAddBookOkButtonStatus = function(noOfBooks){
		if(noOfBooks > 0) {
			$('#btn_ok_mul_add').attr('disabled', false);
		} else {
			$('#btn_ok_mul_add').attr('disabled', true);
		}
	};
	
	var goBack = function(){
		if(Chapter.history.chapterNo == null){
			return;
		}
		
		Chapter.goBack();
		
		//	Reset the variables.
		Chapter.history.chapterNo = null;
		Chapter.history.readingDepth = null;
		
		Toolbar.disableBackButton();
	}
	
	var showAllNotes = function(){
		if($('#doc-viewer-azd').contents().find('#notes_placeholder').css('display') == 'block'){
			return;
		}
		
		$('#doc-viewer-azd').contents().find('#toc_placeholder').hide();
		$('#doc-viewer-azd').contents().find('#notes_placeholder').show();
		
		$('#doc-viewer-azd').contents().find('#content').removeClass('section-active');
		$('#doc-viewer-azd').contents().find('#notes').addClass('section-active');
	}
	
	var showTableOfContent = function(){
		if($('#doc-viewer-azd').contents().find('#toc_placeholder').css('display') == 'block'){
			return;
		}
		
		$('#doc-viewer-azd').contents().find('#notes_placeholder').hide();
		$('#doc-viewer-azd').contents().find('#toc_placeholder').show();
		
		$('#doc-viewer-azd').contents().find('#content').addClass('section-active');
		$('#doc-viewer-azd').contents().find('#notes').removeClass('section-active');
	}
	
	var showLandmarksColumn = function() {
		if($('#doc-viewer-azd').contents().find('#landmark_placeholder').css('display') == 'block'){
			return;
		}
		
		$('#doc-viewer-azd').contents().find('#page-nav_placeholder').hide();
		$('#doc-viewer-azd').contents().find('#landmark_placeholder').show();
		
		$('#doc-viewer-azd').contents().find('#landmark').addClass('section-active');
		$('#doc-viewer-azd').contents().find('#page-nav').removeClass('section-active');
	};
	
	var showPageNavColumn = function(){
		if($('#doc-viewer-azd').contents().find('#page-nav_placeholder').css('display') == 'block'){
			return;
		}
		
		$('#doc-viewer-azd').contents().find('#landmark_placeholder').hide();
		$('#doc-viewer-azd').contents().find('#page-nav_placeholder').show();
		
		$('#doc-viewer-azd').contents().find('#landmark').removeClass('section-active');
		$('#doc-viewer-azd').contents().find('#page-nav').addClass('section-active');
	};
	
	/**
	 * Function to go to note from full-page toc page
	 */
	var goToNote = function(id, chapter){
		var self = this;
		
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			$('#but_toc').removeAttr('checked');
			if(Epub.config.version == 3) {
				if(Epub.config.epub3TocData != ''){
					Epub.setInPageTocDataEpub3();
				}
				else{
					Epub.setInPageTocDataEpub2();
				}
			}
			else {
				Epub.setInPageTocDataEpub2();
			}
			Chapter.goToNote(id, chapter);
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
		Toolbar.showEpub();
	};
	
	var toggleColumnRule = function(display) {
		if(BookPref.config.columns) {
			Columns.toggleColumnRule(display);
		}
	}
	
	var toggleColumn = function(column, noOfCol){
		var self = this;
		
		self.showLoading();
		if(column){
			BookPref.config.columns = true;
			Columns.config.showLastPage = false;
			Columns.config.goToDepth = Flow.config.currentReadingDepth;
			
			Columns.init(noOfCol);
			
			if(Columns.config.displayColumnRule == true) {
				self.toggleColumnRule(true);
			}
		}
		else{
			BookPref.config.columns = false;
			Flow.config.showLastPage = false;
			Flow.config.goToDepth = Columns.config.currentReadingDepth;
			
			Flow.init();
		}
		self.hideLoading();
	}
	
	/**
	 * Function to delete the note
	 * @params noteId Id of the note
	 * @params deleteButton DOM reference of the delete button pressed
	 */
	var deleteNote = function(noteId, deleteButton){
		var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
 		
		if(prompts.confirm(null, AME.getTitle(), AzdMessages.deleteNote)){
			Notes.deleteNoteFromDatabase(noteId);
			Notes.deleteNoteFromFullPage(deleteButton);
		}
	};
	
	var clearTempDir = function () {
		Epub.getExtractLocation();
		var extractLocation = Epub.config.extractLocation;
		var fileObj = getFileObjectFromPath(extractLocation);
		if (fileObj.exists()) {
			deleteFolder(fileObj.path);
		}
	}
	
	var openSearchPage = function () {
		var self = this;
		
		self.hideStatusBar();
		$('#doc-viewer-azd').attr('src', '');
		$('#doc-viewer-azd').attr('src', 'html/azardi2_search_doc.html');
	}
	
	var showStatusBar = function () {
		$('#azd-statusbar').show();
	}
	
	var hideStatusBar = function () {
		$('#azd-statusbar').hide();
	}
	
	var showCatalog = function () {
		var self = this;
		
		// //	Reset the books current position.
		// //TODO: Check if need to call unloadAco function.
		// CdpNav.book.curr_pos = -1;
		// //	Load the catalog.
		// if(CdpDownload.config.isDownloading){
			// self.showStatusBar();
		// }
		Catalog.load(self.getCatalogId());
	}
	
	var openCdpBookmarkPage = function () {
		var params = {
						inn:{
							type:'bookmark'
						},
						out:null
					 };
		window.openDialog("chrome://azardi2/content/azd_page_container.xul", "Bookmarks", "chrome,width=500,height=655, dialog, modal, resizable=no, dependent, centerscreen", params).focus();
    }
    
    var openCdpGlossaryPage = function () {
    	if(Azd.book.show_glossary){
	    	var params = {
	    				 	inn:{
	    				 		type:'glossary'
	    				 	},
	    				 	out:null
	    				 };
			window.openDialog("chrome://azardi2/content/azd_page_container.xul", "Glossary", "chrome,width=500,height=655, dialog, modal, resizable=no, dependent, centerscreen", params).focus();
		}
    }
    
    /**
     * 	This function is needed to get the tooltip text of html page set in the xul iframe.
     */
    
    var fillInHTMLTooltip = function (tipElement) {
    	var retVal = false;
		if (tipElement.namespaceURI == "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul") {
			return retVal;
		}
		const XLinkNS = "http://www.w3.org/1999/xlink";
		var titleText = null;
		var XLinkTitleText = null;
		while (!titleText && !XLinkTitleText && tipElement) {
			if (tipElement.nodeType == Node.ELEMENT_NODE) {
				titleText = tipElement.getAttribute("title");
				XLinkTitleText = tipElement.getAttributeNS(XLinkNS, "title");
			}
			tipElement = tipElement.parentNode;
		}
		var texts = [titleText, XLinkTitleText];
		var tipNode = document.getElementById("aHTMLTooltip");
		for (var i = 0; i < texts.length; ++i) {
			var t = texts[i];
			if (t && t.search(/\S/) >= 0) {
				tipNode.setAttribute("label", t.replace(/\s+/g, " "));
				retVal = true;
			}
		}
		return retVal;
    };
    
    var showMessage = function (message, priority, autohide, buttons) {
    	var self = this;
    	
		var nb = document.getElementById('message_pane');
		
		switch(priority){
			case 'info':
				priority = nb.PRIORITY_INFO_MEDIUM;
				break;
			case 'warning':
				priority = nb.PRIORITY_WARNING_MEDIUM;
				break;
			case 'critical':
				priority = nb.PRIORITY_CRITICAL_MEDIUM;
				break;
		}
		
		if(buttons) {
			nb.appendNotification(message, 'azd_notification', null, priority, buttons)
		}else{
			nb.appendNotification(message, 'azd_notification', null, priority, null)
		}
		
		
		if (autohide) {
		    var tim = {
				notify: function(timer) {
					try {
						nb.removeCurrentNotification();
					} 
					catch (e) {
					}
					tim.timer = null;	// break circular reference
				}
		    };
	
		    var timer = Components.classes["@mozilla.org/timer;1"].
			createInstance(Components.interfaces.nsITimer);
		    tim.timer = timer;	// NOTE circular reference
		    timer.initWithCallback(tim, 10*1000, timer.TYPE_ONE_SHOT);
		}
		// var n = nb.getNotificationWithValue('popup-blocked');
		// if(n) {
			// n.label = message;
		// } else {
			// var buttons = [{
				// label : 'Button',
				// accessKey : 'B',
				// popup : 'blockedPopupOptions',
				// callback : null
			// }];
// 
			// priority = nb.PRIORITY_WARNING_BLOCK;
			// nb.appendNotification(message, 'popup-blocked', null, priority, buttons);
		// }
    } 
    
    var removeAllMessages = function () {
    	var self = this;
    	
		var nb = document.getElementById('message_pane');
    	nb.removeAllNotifications(true)
    }
    
    // var bindArrowKeys = function(element) {
		// $(element).keydown(function (e) {
			// var keyCode = e.keyCode || e.which,
				// arrow = {left: 37, up: 38, right: 39, down: 40 };
// 			
			// switch (keyCode) {
				// case arrow.left:
					// // parent.AME.previousPage();
					// e.preventDefault();
					// break;
				// case arrow.up:
					// // parent.AME.previousPage();
					// e.preventDefault();
					// break;
				// case arrow.right:
					// // parent.AME.nextPage();
					// e.preventDefault();
					// break;
				// case arrow.down:
					// // parent.AME.nextPage();
					// e.preventDefault();
					// break;
			// }
		// });
    // }
//     
    // var unbindArrowKeys = function(element) {
    	// $(element).unbind('keydown');
    // }
    
    var showBookInfoDialog = function(bookId) {
    	var self = this;
    	
    	if(typeof bookId == 'undefined') {
    		bookId = Library.getClickedBook();
    	}
		
		var bookInfo = self.getBookInfo(bookId);
		
		var params = {
			inn : {
				bookInfo : bookInfo
			},
			out : {
				out : null
			}
		};
		window.openDialog("chrome://azardi2/content/azd_book_information.xul", "Book Information", "chrome,width=500,height=400, dialog, modal, resizable=yes, dependent, centerscreen", params).focus();
    }
    
    var getBookInfo = function(bookId) {
    	var self = this;
    	
    	var bookInfo = new Array();
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if(Library.config.bookMetadata[i]['id'] === +bookId){
				bookInfo['id'] = Library.config.bookMetadata[i]['id'],
				bookInfo['type'] = Library.config.bookMetadata[i]['type'],
				bookInfo['uid'] = Library.config.bookMetadata[i]['uid'],
				bookInfo['title'] = Library.config.bookMetadata[i]['title'],
				bookInfo['author'] = Library.config.bookMetadata[i]['author'],
				bookInfo['publisher_identifier'] = Library.config.bookMetadata[i]['publisher_identifier'],
				bookInfo['publish_date'] = Library.config.bookMetadata[i]['publish_date'];
				bookInfo['created_date'] = new Date(parseInt(Library.config.bookMetadata[i]['created_date'])).toLocaleDateString();
				bookInfo['description'] = Library.config.bookMetadata[i]['description'];
				bookInfo['location'] = Library.config.bookMetadata[i]['location'];
				bookInfo['cover_image'] = Library.config.bookMetadata[i]['cover_image'];
				bookInfo['cover_type'] = Library.config.bookMetadata[i]['cover_type'];
				bookInfo['version'] = Library.config.bookMetadata[i]['version'];
				break;
			}
		}
		return bookInfo;
    }
    
    /**
     * Function to navigate from page-nav
     */
    var goToFromPageNav = function(referenceLocation) {
    	var self = this;
    	
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			$('#but_landmark').removeAttr('checked');
			if(Epub.config.version == 3) {
				if(Epub.config.epub3TocData != ''){
					Epub.setInPageTocDataEpub3();
				}
				else{
					Epub.setInPageTocDataEpub2();
				}
			}
			else {
				Epub.setInPageTocDataEpub2();
			}
			
			Chapter.goToFromPageNav(referenceLocation);
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
		Toolbar.showEpub();
    };
    
    /**
     * Function to navigate from landmark
     */
    var goToFromLandmark = function (referenceLocation) {
    	var self = this;
    	
		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
			$('#but_landmark').removeAttr('checked');
			if(Epub.config.version == 3) {
				if(Epub.config.epub3TocData != ''){
					Epub.setInPageTocDataEpub3();
				}
				else{
					Epub.setInPageTocDataEpub2();
				}
			}
			else {
				Epub.setInPageTocDataEpub2();
			}
			
			Chapter.goToFromLandmark(referenceLocation);
			$('#doc-viewer-azd').unbind('DOMContentLoaded');
		});
		
		$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
		Toolbar.showEpub();
    };
    
    /**
     * This function initiates navigation from toc
     * @params referenceLocation Destination location
     * @params tocType Type of toc used to initiate the navigation side|full-page
     */
    var goToFromToc = function(referenceLocation, tocType){
    	var self = this;
    	
    	if(tocType == 'side') {
    		Chapter.goToFromToc(referenceLocation, tocType);
    	} else {
    		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
    			$('#but_toc').removeAttr('checked');
    			if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						//Set fly-out toc data in epub3
						Epub.setInPageTocDataEpub3();
					}
					else{
						//Set fly-out toc data in epub2
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					//Set fly-out toc data in epub2
					Epub.setInPageTocDataEpub2();
				}
				Chapter.goToFromToc(referenceLocation, tocType);
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			//Show epub toolbar
			Toolbar.showEpub();
    	}
    };
    
    /**
     * Function to initilize the sequence to go to cover page
     * @params tocType Type of the toc navigating from
     */
    var goToCoverPage = function(tocType) {
    	var self = this;
    	
    	if(tocType == 'side') {
    		Chapter.goToCoverPage(tocType);
    	} else {
    		$('#doc-viewer-azd').bind('DOMContentLoaded', function(e){
    			$('#but_toc').removeAttr('checked');
    			if(Epub.config.version == 3) {
					if(Epub.config.epub3TocData != ''){
						Epub.setInPageTocDataEpub3();
					}
					else{
						Epub.setInPageTocDataEpub2();
					}
				}
				else {
					Epub.setInPageTocDataEpub2();
				}
				Chapter.goToCoverPage(tocType);
				$('#doc-viewer-azd').unbind('DOMContentLoaded');
			});
			$('#doc-viewer-azd').attr('src', 'html/azardi2_default_epub_doc.html');
			Toolbar.showEpub();
    	}
    };
    
    var previousPageUsingKey = function () {
    	var self = this;
    	
    	self.previousPage();
    }
    
    var nextPageUsingKey = function () {
    	var self = this;
    	
    	self.nextPage();
    }
    
    var previousChapterUsingKey = function() {
    	var self = this;
    	
    	self.previousChapter();
    };
    
    var nextChapterUsingKey = function() {
    	var self = this;
    	
    	self.nextChapter();
    };
    
    var openAboutWindow = function () {
    	var self = this;
    	
    	var params = {
			// inn : {
				// bookInfo : bookInfo
			// },
			// out : {
				// out : null
			// }
		};
		window.openDialog("chrome://azardi2/content/about.xul", "About", "chrome,width=500,height=330, dialog, resizable=no, dependent, centerscreen", params).focus();
    }
    
    /**
     * Open image in new viewport
     */
    var openImage = function (src) {
    	var self = this;
    	
    	var params = {
    		inn : {
				src : src,
				type : 'image'
			},
			out : {
				out : null
			}
		};
    	
    	window.openDialog("chrome://azardi2/content/azardi_external_container.xul", "External Container", "chrome, width=1000, height=700, dialog, resizable=yes, centerscreen", params).focus();
    };
    
    var checkToken = function () {
		var self = this;

		var fileObj = getFileObject(self.getDbFilePath());
		table_name = 'System';
		var conditions = new Array(['key', 'token', 'TEXT']);
		var values = ['value'];
		
		var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
		var value;
		while (statement.executeStep()) {
			value = statement.row.value
		}
		statement.finalize();
		
		if(typeof value == 'undefined')
			return false;
		else{
			config.readerToken = value;
			return true;
		}
	};
	
	var resetBookPreferences = function() {
		BookPref.init();
		BookPref.set();
		
		if(BookPref.config.columns){
			Columns.init(BookPref.config.noOfCols);
			Columns.toggleColumnRule(BookPref.config.displayColumnRule);
		} else {
			Flow.init();
		}
	};
	
	var playAudio = function(target) {
		if($('#but_audio_control').attr('disabled') != 'true') {
			if($('#but_audio_control').attr('checked') != 'true') {
				$('#but_audio_control').click();
			}
			document.getElementById('doc-viewer-azd').contentWindow.playAudio($(target));
		}
	};
	
	var stopAudio = function(target) {
		if($('#but_audio_control').attr('disabled') != 'true') {
			if($('#but_audio_control').attr('checked') != 'true') {
				$('#but_audio_control').click();
			}
			document.getElementById('doc-viewer-azd').contentWindow.stopAudio($(target));
		}
	};
	
	/**
	 * Function to handle the play event
	 */
	var playing_handle = function() {
		$('#but_audio_play').addClass('playing');
		// show pause button
		$('#but_audio_play').attr('image', 'icons/ico_audio-pause.png');
	};
	
	/**
	 * Function to handle the pause event
	 */
	var paused_handle = function() {
		$('#but_audio_play').removeClass('playing');
		// show playing button
		$('#but_audio_play').attr('image', 'icons/ico_audio-play.png');
	};
	
	/**
	 * Initilizes the keyboard events.
	 */
	var setupKeyEvent = function() {
		$(document).keydown(function(event) {
			switch(event.which){
				case 16:
					//	Shift key
					config.isShiftPressed = true;
					break;
				case 17:
					//	Ctrl key
					config.isCtrlPressed = true;
					break;
				case 32:
					//	Spacebar
					if(config.isShiftPressed) {
						AME.stopAudio($('#but_audio_stop'));
						event.preventDefault();
					} else if(config.isCtrlPressed) {
						AME.playAudio($('#but_audio_play'));
						event.preventDefault();
					}
					break;
				default:
			}
		});
		
		$(document).keyup(function(event) {
			switch(event.which){
				case 16:
					//	Shift key
					config.isShiftPressed = false;
					break;
				case 17:
					//	Ctrl key
					config.isCtrlPressed = false;
					break;
				default:
			}
		});
	};
	
    return {
		init: init,
		setBootupPreferences: setBootupPreferences,
		// getRenewCdpUrl: getRenewCdpUrl,
		// getRenewCatUrl: getRenewCatUrl,
		getDBVersion: getDBVersion,
		getDefaultDirPath: getDefaultDirPath,
		getDbFilePath: getDbFilePath,
		getCdpDbFilePath: getCdpDbFilePath,
		getSearchDbFilePath: getSearchDbFilePath,
		getTempDownloadPath: getTempDownloadPath,
		// getCfsLocation: getCfsLocation,
		getCatalogId: getCatalogId,
		getReaderToken: getReaderToken,
		getAppName: getAppName,
		// createDefaultDbTables: createDefaultDbTables,
		showLibrary: showLibrary,
		closeLibrary: closeLibrary,
		changeLibraryView: changeLibraryView,
		toggleShelf: toggleShelf,
		showShelf: showShelf,
		closeShelf: closeShelf,
		deleteShelf: deleteShelf,
		windowTitle: windowTitle,
		resetAddBookForm: resetAddBookForm,
		showAddBookForm: showAddBookForm,
		closeAddBookForm: closeAddBookForm,
		showUserPrefForm: showUserPrefForm,
		closeUserPrefForm: closeUserPrefForm,
		saveUserPrefs: saveUserPrefs,
		resetCreateShelfForm: resetCreateShelfForm,
		showCreateShelfForm: showCreateShelfForm,
		closeCreateShelfForm: closeCreateShelfForm,
		addBook: addBook,
		deleteBook: deleteBook,
		addBookToShelf: addBookToShelf,
		createShelf: createShelf,
		selectFile: selectFile,
		sortLibrary: sortLibrary,
		enterBookInformation: enterBookInformation,
		setCoverThumbnail: setCoverThumbnail,
		getCoverThumbDataUri: getCoverThumbDataUri,
		openBook: openBook,
		openBookFromCatalog: openBookFromCatalog,
		showLoading: showLoading,
		hideLoading: hideLoading,
		toggleFullPageToc: toggleFullPageToc,
		toggleLandmark: toggleLandmark,
		nextPage: nextPage,
		previousPage: previousPage,
		nextChapter: nextChapter,
		previousChapter: previousChapter,
		changeFontSize: changeFontSize,
//		changeParagraphSpace: changeParagraphSpace,
		changeLineSpace: changeLineSpace,
		changeMarginSpace: changeMarginSpace,
		changeBackgroundColor: changeBackgroundColor,
		toggleJustify: toggleJustify,
		toggleHyphen: toggleHyphen,
		resizeWindow: resizeWindow,
		openNotesDialog: openNotesDialog,
		closeAddNotesDialog: closeAddNotesDialog,
		saveNotes: saveNotes,
		openEditNotesDialog: openEditNotesDialog,
		closeEditNotesDialog: closeEditNotesDialog,
		saveEditedNotes: saveEditedNotes,
		changeShelfOkButtonStatus: changeShelfOkButtonStatus,
		changeAddBookOkButtonStatus: changeAddBookOkButtonStatus,
		changeMulAddBookOkButtonStatus: changeMulAddBookOkButtonStatus,
		goBack: goBack,
		showAllNotes: showAllNotes,
		showTableOfContent: showTableOfContent,
		goToNote: goToNote,
		toggleColumn: toggleColumn,
		deleteNote: deleteNote,
		clearTempDir: clearTempDir,
		openSearchPage: openSearchPage,
		showStatusBar: showStatusBar,
		hideStatusBar: hideStatusBar,
		getTitle: getTitle,
		showCatalog: showCatalog,
		openCdpBookmarkPage: openCdpBookmarkPage,
		openCdpGlossaryPage: openCdpGlossaryPage,
		registerApp: registerApp,
		fillInHTMLTooltip: fillInHTMLTooltip,
		copyDefaultDbTables: copyDefaultDbTables,
		checkDbVersion: checkDbVersion,
		showMessage: showMessage,
		removeAllMessages: removeAllMessages,
		// bindArrowKeys:bindArrowKeys,
		// unbindArrowKeys: unbindArrowKeys,
		showBookInfoDialog: showBookInfoDialog,
		getBookInfo: getBookInfo,
		scaleSize: scaleSize,
		goToFromToc: goToFromToc,
		goToFromLandmark: goToFromLandmark,
		checkUpdates: checkUpdates,
		previousPageUsingKey: previousPageUsingKey,
		nextPageUsingKey: nextPageUsingKey,
		openAboutWindow: openAboutWindow,
		openImage: openImage,
		goToCoverPage: goToCoverPage,
		goToFromPageNav: goToFromPageNav,
		showLandmarksColumn: showLandmarksColumn,
		showPageNavColumn: showPageNavColumn,
		addMultipleBooks: addMultipleBooks,
		previousChapterUsingKey: previousChapterUsingKey,
		nextChapterUsingKey: nextChapterUsingKey,
		checkToken: checkToken,
		downloadCat: downloadCat,
		toggleColumnRule: toggleColumnRule,
		playAudio: playAudio,
		stopAudio: stopAudio,
		playing_handle: playing_handle,
		paused_handle: paused_handle,
		resetBookPreferences: resetBookPreferences,
		toggleAudioControls: toggleAudioControls,
		setupKeyEvent: setupKeyEvent,
		openBookAtFirst: openBookAtFirst
    };
}();
