/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

var BookMetadata = function(){
	
	/**
	 * Private properties and methods
	 */
	
	var config = {
		bookMetadata: []
	}
	/**
	 * Public properties and methods
	 */
	
	var clear = function(){
		config.bookMetadata.splice(0, config.bookMetadata.length);
	}
	
	var add = function(bookMetadata){
		
		var fileObj = getFileObject(AME.getDbFilePath());
	    var table_name = 'library_master';
	    var values = new Array(['created_date', bookMetadata.created_date, 'TEXT'],
		   					  ['title', bookMetadata.title, 'TEXT'],
		   					  ['author', bookMetadata.author, 'TEXT'],
		  					  ['publisher_identifier', bookMetadata.publisher_identifier, 'TEXT'],
		 					  ['publish_date', bookMetadata.publish_date, 'TEXT'],
		 					  ['description', bookMetadata.description, 'TEXT'],
		 					  ['location', bookMetadata.location, 'TEXT'],
		 					  ['uid', bookMetadata.uid, 'TEXT'],
		 					  ['type', bookMetadata.type, 'TEXT'],
		 					  ['cover_image', bookMetadata.cover_image, 'TEXT'],
		 					  ['cover_type', bookMetadata.cover_type, 'TEXT'],
		 					  ['version', bookMetadata.version, 'TEXT']);
	    insertRowIntoDatabase(fileObj, values, table_name);
	}
	
	var getAll = function(){
		var self = this;
		
		self.clear();
		
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id', 'type', 'uid', 'created_date', 'title', 'author', 'publisher_identifier', 'publish_date', 'description', 'location', 'cover_image', 'cover_type', 'version'];
		var tableName = 'library_master';
		var statement = getValuesFromDatabase(fileObj, values, tableName);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['type'] = statement.row.type;	
			data['uid'] = statement.row.uid;
			data['created_date'] = statement.row.created_date;
			data['title'] = statement.row.title;
			data['author'] = statement.row.author;
			data['publisher_identifier'] = statement.row.publisher_identifier;
			data['publish_date'] = statement.row.publish_date;
			data['description'] = statement.row.description;
			data['location'] = statement.row.location;
			data['cover_image'] = statement.row.cover_image;
			data['cover_type'] = statement.row.cover_type;
			data['version'] = statement.row.version;
			config.bookMetadata.push(data);
		}
		statement.finalize();
		
		return config.bookMetadata;
	}
	
	var getShelfBooks = function(shelfId){
		var self = this;
		
		self.clear();
		
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()) {
			return;
		}
		var queryStatement = 'select id, type, uid, created_date, title, author, publisher_identifier, publish_date, description, location, cover_image, cover_type, version from library_master'+
							 ' where id in (select lib_id from shelf_document_join where shelf_id=?1);';
		var condition = new Array([shelfId, 'INTEGER']);
		
		var statement = getValuesFromDatabaseJoin(fileObj, queryStatement, condition);
		
		while(statement.executeStep()){
			var data = new Array();
			data['id'] = statement.row.id;
			data['type'] = statement.row.type;	
			data['uid'] = statement.row.uid;
			data['created_date'] = statement.row.created_date;
			data['title'] = statement.row.title;
			data['author'] = statement.row.author;
			data['publisher_identifier'] = statement.row.publisher_identifier;
			data['publish_date'] = statement.row.publish_date;
			data['description'] = statement.row.description;
			data['location'] = statement.row.location;
			data['cover_image'] = statement.row.cover_image;
			data['cover_type'] = statement.row.cover_type;
			data['version'] = statement.row.version;
			config.bookMetadata.push(data);
		}
		statement.finalize();
		
		return config.bookMetadata;
	}
	
	var getBookId = function(bookTitle) {
		var self = this;
		
		var fileObj = getFileObject(AME.getDbFilePath());
		if (!fileObj.exists()) {
			return;
		}
		var values = ['id'];
		var tableName = 'library_master';
		var conditions = new Array(['title', bookTitle, 'TEXT']);
		var statement = getValuesFromDatabase(fileObj, values, tableName, conditions);
		var id;
		while(statement.executeStep()){
			id = statement.row.id;
		}
		statement.finalize();
		
		return id;
	}
	
	return{
		clear: clear,
		add: add,
		getAll: getAll,
		getShelfBooks: getShelfBooks,
		getBookId: getBookId
	}
}();