/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary for loading particular chapter.
 */
var Chapter = function(){
	var config = {
		openAtFirst: false,
		currentChapter: 0,
		// containsMathMl: false,
		// containsSVG: false,
		standAlonePage: false,
		XHTML_NS: "http://www.w3.org/1999/xhtml",
		azardi_presentation: null,
		azardi_width: '',
		azardi_height: '',
		azardi_margins: -1,
		azardi_toc: null,
		oldReadingDepth: null,
		orig_width: null,
		orig_height: null,
		toolbar_height: 30
	};
	
	var history = {
		chapterNo: null,
		readingDepth: null
	};
	
	/**
	 * Initilize the chapter
	 */
	var init = function(){
		try{
		var self = this;
		
		//Do not remove the line, used to calculate correct innerHeight
		config.toolbar_height = parseInt(window.innerHeight) - parseInt(document.defaultView.getComputedStyle(document.getElementById("doc-viewer-azd"), null).getPropertyValue("height"));
		
		if(Epub.config.version == 3) {
			if(Epub.config.epub3TocData != ''){
				//Set fly-out toc for epub3
				Epub.setInPageTocDataEpub3();
			}
			else{
				//Set fly-out toc for epub2
				Epub.setInPageTocDataEpub2();
			}
		}
		else {
			//Set fly-out toc for epub2
			Epub.setInPageTocDataEpub2();
		}
		
//		self.load();
		//Get the book preferences from the database
		var bookPref = BookPref.getFromDB();
		if(typeof bookPref == 'undefined'){
			BookPref.init();	// It overrides book pref with user pref.
			
			// window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			
			self.load();
			
			if(config.azardi_presentation == 'fixed') {
				self.unbindResize();
				if(config.azardi_width && config.azardi_height) {
					if(config.azardi_margins != -1) {
						window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
					} else {
						window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				} else if(config.azardi_width) {
					if(config.azardi_margins != -1) {
						window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
					} else {
						window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = BookPref.config.openHeight;
				} else if(config.azardi_height){
					window.innerWidth = BookPref.config.openWidth;
					window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
					
				} else {
					window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
				}
			} else if(config.azardi_presentation == 'fullscreen') {
				self.unbindResize();
				width = screen.width || BookPref.config.openWidth;
				height = screen.height || BookPref.config.openHeight;
				window.resizeTo(width, height);
				window.moveTo(0, 0);
			} else if(config.azardi_presentation == null) {
				self.unbindResize();
				self.bindResize();
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
			
			config.orig_width = BookPref.config.openWidth;
			config.orig_height = BookPref.config.openHeight;
			// window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			Justify.getParagraphsDefaultAlignment();
			// self.checkForMathMl();
			// self.checkForSVG();
			//	Hyphenation creates problem if page contains mathml.
			// if(!config.containsMathMl && !config.containsSVG){
				// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			// }
			BookPref.set();
		
			if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
				Columns.config.showLastPage = false;
			}
			else{
				Flow.config.showLastPage = false;
			}
			self.processInternalLinks();
			self.processParagraphs();
			Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//			Notes.highlight();
			// if(!config.containsMathMl && !config.containsSVG){
				// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			// }
			self.loadAudio();
			self.loadVideo();
			self.loadImages();
			
			//	Here we can delete the extracted folder.
			deleteFolder(getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder]));
		}
		else{
			BookPref.initWithSavedPreferences(bookPref);	// It overrides book pref with saved book prefs.
			
			// window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			
			if(!config.openAtFirst) {
				config.currentChapter = BookPref.config.currentChapter;
			}
			
			if(String(config.currentChapter).indexOf('azd-outofspine-') == -1){
				config.standAlonePage = false;
				self.load();
			} else {
				var fileLocation = config.currentChapter.replace('azd-outofspine-', '');
				config.standAlonePage = true;
				self.loadStandAlonePage(fileLocation);
			}
			
			if(config.azardi_presentation == 'fixed') {
				self.unbindResize();
				if(config.azardi_width && config.azardi_height) {
					if(config.azardi_margins != -1) {
						window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
					} else {
						window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				} else if(config.azardi_width) {
					if(config.azardi_margins != -1) {
						window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
					} else {
						window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = BookPref.config.openHeight;
				} else if(config.azardi_height){
					window.innerWidth = BookPref.config.openWidth;
					window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
					
				} else {
					window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
				}
			} else if(config.azardi_presentation == 'fullscreen') {
				self.unbindResize();
				width = screen.width || BookPref.config.openWidth;
				height = screen.height || BookPref.config.openHeight;
				window.resizeTo(width, height);
				window.moveTo(0, 0);
			} else if(config.azardi_presentation == null) {
				self.unbindResize();
				self.bindResize();
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
			
			config.orig_width = BookPref.config.openWidth;
			config.orig_height = BookPref.config.openHeight;
			// window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			// self.load();
			Justify.getParagraphsDefaultAlignment();
			// self.checkForMathMl();
			// self.checkForSVG();
			//	Hyphenation creates problem if page contains mathml.
			// if(!config.containsMathMl && !config.containsSVG){
				// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			// }
			BookPref.set();
			
			if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
				Columns.config.showLastPage = false;
				
				Columns.config.goToDepth = BookPref.config.readingDepth;
					
			}
			else{
				Flow.config.showLastPage = false;
				
				//	No reading depth for non column mode.
				
			}
			
			self.processInternalLinks();
			self.processParagraphs();
			Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//			Notes.highlight();
			// if(!config.containsMathMl && !config.containsSVG){
				// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			// }
			self.loadAudio();
			self.loadVideo();
			self.loadImages();
		}
		// Toolbar.showEpub();
		
		document.getElementById('doc-viewer-azd').contentWindow.initAIE();
		}catch(e){
		}
	}
	
	/**
	 * Loads the current chapter
	 */
	var load = function(){
		var self = this;
		
		//Get spine of current chapter
		var idref = Epub.getCurrentChapterFromSpine();
		//Get fixed layout details of current chapter
		var chapterPresentation = Epub.getChapterPresentationDetails(idref);
		self.setChapterPresentation(chapterPresentation);
		//Get media type of current chapter
		var mediaType = Epub.getChapterMediaType(idref);
		//Get properties of current chapter
		var properties = Epub.getChapterProperties(idref);
		//Get href of current chapter
		var href = Epub.getChpaterHref(idref);
		//Get html data of current chapter
		var data = Epub.getChapterData(idref);
		//Get css references of current chapter
		var cssRef = Epub.getChapterCssRef(idref);
		//Get js references of current chapter
		var jsRef = Epub.getChapterJsRef(idref);
		//Get media overlay of current chapter
		var mediaOverlay = Epub.getChapterOverlay(idref);
		
		//Initilize the reader's audio player if chapter has smil data associated with it
		self.initAudioPlayer(mediaOverlay);
		
		//Remove old css references
		self.removeCssReference();
		// self.removeJsReference();
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html('');

		if(cssRef != ''){
			//Set new css references
			self.setCssReference(cssRef);
		}
		
		//If current chapter contains mathml then set the stix fonts
		if(properties.search(/mathml/i) >= 0) {
			self.setStixFont();
		}
		
		if(jsRef != ''){
			self.setJsReference(jsRef);
		}
		
		if(mediaType.split('/')[0] == 'image') {
			self.loadImageAsPage(data);
		} else if(mediaType == 'application/x-shockwave-flash') {
			self.loadSwfAsPage(href);
		} else if(mediaType == 'application/pdf') {
			self.loadPdfAsPage(href);
		} else {
			$('#doc-viewer-azd').contents().find('#custom-style-igp').html(data);
		}
		
		if(mediaOverlay != '') {
			Smil.init(mediaOverlay);
		}
		
		if(config.azardi_toc == 'false') {
			self.hideToc();
			Toolbar.disableLandmarkButton();
		} else {
			self.showToc();
			if(Epub.config.version == 3) {
				Toolbar.enableLandmarkButton();
			}
		}
	};
	
	/**
	 * Set the css for stix fonts
	 */
	var setStixFont = function() {
		var styleTag = '<link rel="stylesheet" type="text/css" href="../css/STIXfonts.css" id="stixCss" />';
		$('#doc-viewer-azd').contents().find('head').append(styleTag);
	};
	
	/**
	 * Hide flyout toc
	 */
	var hideToc = function() {
		$('#doc-viewer-azd').contents().find('.toc-azd').hide();
		Toolbar.disableTocButton();
	}
	
	/**
	 * Show flyout toc
	 */
	var showToc = function() {
		$('#doc-viewer-azd').contents().find('.toc-azd').show();
		Toolbar.enableTocButton();
	}
	
	/**
	 * Set fixed layout properties for current chapter
	 * @params chapterPresentation Fixed layout details of current chapter
	 */
	var setChapterPresentation = function(chapterPresentation) {
		var self = this;
		
		config.azardi_presentation = (chapterPresentation['azardi-presentation'] != '') ? chapterPresentation['azardi-presentation'] : null;
		config.azardi_width = chapterPresentation['azardi-width'];
		config.azardi_height = chapterPresentation['azardi-height'];
		config.azardi_margins = (chapterPresentation['azardi-margins'] != '') ? parseInt(chapterPresentation['azardi-margins']) : -1;
		config.azardi_toc = (chapterPresentation['azardi-toc'] != '') ? chapterPresentation['azardi-toc'] : null;
	};
	
	/**
	 * Loads the .pdf file as a page
	 * @params href href path of .pdf file
	 */
	var loadPdfAsPage = function(href) {
		var self = this;
		
		var id = href.split('/').pop();
		var pdfUrl = '';
		var pdfObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, id]);
		if(pdfObj.exists()){
			pdfUrl = getURL(pdfObj);
		}
		
		var data = pdfUrl + '?#scrollbar=1&toolbar=1&navpanes=0&view=FitV&statusbar=0';
		
		var pdfPage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('div'))
						.addClass('azardi-pdf-page')
						.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('object'))
							.attr('id', 'pdf')
							.attr('data', data)
							.attr('type', 'application/pdf')
							// .attr('width', '550')
							// .attr('height', '400')
							.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('p'))
								.html('This PDF cannot be display. You must install Adobe Reader.')
							)
						)
						
		$('#doc-viewer-azd').contents().find('#custom-style-igp').append(pdfPage);
	};
	
	/**
	 * Loads the .swf file as a page
	 * @params href path of the .swf file
	 */
	var loadSwfAsPage = function(href) {
		var self = this;
		
		var id = href.split('/').pop();
		var swfUrl = '';
		var swfObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, id]);
		if(swfObj.exists()){
			swfUrl = getURL(swfObj);
		}
		
		var flashPage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('object'))
							.attr('classid', 'clsid:d27cdb6e-ae6d-11cf-96b8-444553540000')
							.attr('width', '550')
							.attr('height', '400')
							.attr('id', id)
							.attr('align', 'middle')
							.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('param'))
								.attr('name', id)
								.attr('value', swfUrl)
							)
							.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('a'))
								.attr('href', 'http://www.adobe.com/go/getflash')
								.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('img'))
									.attr('src', 'http://www.adobe.com/images/shared/download_buttons/get_flash_player.gif')
									.attr('alt', 'Get Adobe Flash player')
								)
							)
		$('#doc-viewer-azd').contents().find('#custom-style-igp').append(flashPage);
	};
	
	/**
	 * Loads the image as page
	 * @params imageData Datauri of the image to be used as a page
	 */
	var loadImageAsPage = function(imageData) {
		var self = this;
		
		var imagePage = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('div'))
							.addClass('azardi-image-page')
							.append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('img'))
								.attr('src', imageData)
							)
		
		$('#doc-viewer-azd').contents().find('#custom-style-igp').append(imagePage);
	};
	
	var loadPreviewPage = function() {
		var self = this;
		
		var idref = 'preview';
		var href = Epub.getChpaterHref(idref);
		var data = Epub.getChapterData(idref);
		var cssRef = Epub.getChapterCssRef(idref);
		var jsRef = Epub.getChapterJsRef(idref);
		
		self.removeCssReference();
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html('');

		if(cssRef != ''){
			self.setCssReference(cssRef);
		}
		
		if(jsRef != ''){
			self.setJsReference(jsRef);
		}
		
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html(data);
		self.showToc();
		if(Epub.config.version == 3) {
			Toolbar.enableLandmarkButton();
		}
	};
	
	/**
	 * This function is used to load the chapter which are not in spine order.
	 * @params fileLocation href of the file to be loaded
	 */
	var loadStandAlonePage = function(fileLocation) {
		try{
		var self = this;
		
		//Get the presentation details of current chapter
		var chapterPresentation = Epub.getChapterPresentationDetailsFromFileLocation(fileLocation);
		self.setChapterPresentation(chapterPresentation);
		//Get media type of current chapter
		var mediaType = Epub.getChapterMediaTypeFromFileLocation(fileLocation);
		//Get properties of the current chapter
		var properties = Epub.getChapterPropertiesFromFileLocation(fileLocation);
		//Get html data of the current chapter
		var data = Epub.getChapterDataFromFileLocation(fileLocation);
		//Get the css references of the current chapter
		var cssRef = Epub.getChapterCssRefFromFileLocation(fileLocation);
		//Get the js references of the current chapter
		var jsRef = Epub.getChapterJsRefFromFileLocation(fileLocation);
		
		//Remove the old css references
		self.removeCssReference();
		// self.removeJsReference();
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html('');

		if(cssRef != ''){
			self.setCssReference(cssRef);
		}
		
		if(properties.search(/mathml/i) >= 0) {
			self.setStixFont();
		}
		
		if(jsRef != ''){
			self.setJsReference(jsRef);
		}
		
		if(mediaType.split('/')[0] == 'image') {
			self.loadImageAsPage(data);
		} else if(mediaType == 'application/x-shockwave-flash') {
			self.loadSwfAsPage(fileLocation);
		} else if(mediaType == 'application/pdf') {
			self.loadPdfAsPage(fileLocation);
		} else {
			$('#doc-viewer-azd').contents().find('#custom-style-igp').html(data);
		}
		
		if(config.azardi_toc != null) {
			self.hideToc();
		} else {
			self.showToc();
		}
		} catch(e) {
			
		}
	};
	
	/**
	 * Process all the paragraphs of the chapter.
	 * Assign a custom notes class to each paragraph, so it can be used while creating notes.
	 */
	var processParagraphs = function(){
		var self = this;
		
		var $p = $('#doc-viewer-azd').contents().find('#custom-style-igp p');
		$p.each(function(index){
			$(this).addClass('azd-notes-paragraph-'+index);
		});
	};
	
	/**
	 * Process the links in the chapter
	 */
	var processInternalLinks = function(){
		var self = this;
		
		var $links = $('#doc-viewer-azd').contents().find('#custom-style-igp a');
		$links.each(function(){
			var linkReference = $(this).attr('href');
			if(typeof linkReference != 'undefined'){
				if(linkReference.indexOf('http://') == 0 || linkReference.indexOf('https://') == 0){
					$(this).bind('click', function(event){
						event.preventDefault();
						window.open(linkReference, "IEV", "chrome, width=1000, height=700, resizable=yes, centerscreen, scrollbars=yes").focus();
					});
				}
				else if(linkReference != 'javascript: void(0);'){
					if(linkReference.split('#')[0] != ''){
						$(this).bind('click', function(event){
							event.preventDefault();
							self.goToFromInternalLinks(linkReference);
						});
					}
					else{
						$(this).bind('click', function(event){
							event.preventDefault();
							if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
								Columns.goToInternalLinkReference(linkReference.split('#')[1]);	
							}
							else{
								Flow.goToInternalLinkReference(linkReference.split('#')[1]);
							}
						});
					}
				}
			}
		});
	}
	
	/**
	 * Removes old css references from the page
	 */
	var removeCssReference = function(){
		$('#doc-viewer-azd').contents().find('style').each(function(){
			$(this).remove();
		});
		
		//Remove the stix fonts as well
		$('#doc-viewer-azd').contents().find('#stixCss').remove();
	};
	
	var removeJsReference = function() {
		return;
		$('#doc-viewer-azd').contents().find('script').each(function(){
			$(this).remove();
		});
	}
	
	/**
	 * Sets the new css references
	 * @params cssRef css references for the current page
	 */
	var setCssReference = function(cssRef){
		for(var i = 0; i < cssRef.length; i++){
			var cssData = Epub.getCssData(cssRef[i]);
			
			var styleTag = '<style type="text/css">' + cssData + '</style>';
			$('#doc-viewer-azd').contents().find('head').append(styleTag);
		}
	};
	
	var setJsReference = function(jsRef) {
		for(var i = 0; i < jsRef.length; i++){
			if(typeof jsRef[i] != 'undefined') {
				var jsData = Epub.getJsData(jsRef[i]);
				var scriptTag = document.createElementNS(config.XHTML_NS, 'script');
				var scriptContent = document.createTextNode(jsData);
				scriptTag.appendChild(scriptContent);
				document.getElementById('doc-viewer-azd').contentWindow.document.getElementsByTagName('head')[0].appendChild(scriptTag);
			}
		}
	}
	
	/**
	 * Function to go to next viewport/chapter
	 */
	var next = function(){
		var self = this;
		
		document.getElementById('doc-viewer-azd').contentWindow.initPageVariables();
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			var xOffset = document.getElementById('doc-viewer-azd').contentWindow.config.xOffset;
			var documentWidth = document.getElementById('doc-viewer-azd').contentWindow.config.documentWidth;
			var currentWidth = document.getElementById('doc-viewer-azd').contentWindow.config.currentWidth;
			
			if(documentWidth == (currentWidth + xOffset)){
				//Go to next chapter
				if(config.currentChapter < Epub.config.spineList.length - 1 && !config.standAlonePage){
					history.chapterNo = null;
					history.readingDepth = null;
					Toolbar.disableBackButton();
					
					config.currentChapter += 1;
					AME.showLoading();
					self.load();
					
					if(config.azardi_presentation == 'fixed') {
						self.unbindResize();
  						
						if(config.azardi_width && config.azardi_height) {
							if(config.azardi_margins != -1) {
								window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
							} else {
								window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
							}
							window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
						} else if(config.azardi_width) {
							if(config.azardi_margins != -1) {
								window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
							} else {
								window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
							}
							window.innerHeight = BookPref.config.openHeight;
						} else if(config.azardi_height){
							window.innerWidth = BookPref.config.openWidth;
							window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
							
						} else {
							window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
						}
					} else if(config.azardi_presentation == 'fullscreen') {
						self.unbindResize();
						
						width = screen.width || BookPref.config.openWidth;
						height = screen.height || BookPref.config.openHeight;
						window.resizeTo(width, height);
						window.moveTo(0, 0);
					} else if(config.azardi_presentation == null) {
						self.unbindResize();
						self.bindResize();
						window.resizeTo(config.orig_width, config.orig_height);
					}
					
					Justify.getParagraphsDefaultAlignment();
					// self.checkForMathMl();
					// self.checkForSVG();
					//	Hyphenation creates problem if page contains mathml.
					// if(!config.containsMathMl && !config.containsSVG){
						// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
					// }
					BookPref.set();
					Columns.config.showLastPage = false;
					Flow.config.showLastPage = false;
					self.processInternalLinks();
					self.processParagraphs();
					Notes.getNotes(Library.config.currentBookId, config.currentChapter);
					// Notes.highlight();
					// if(!config.containsMathMl && !config.containsSVG){
						// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
					// }
					self.loadAudio();
					self.loadVideo();
					self.loadImages();
				}
				else{
					//	TODO: Disable the next button here.
				}
				document.getElementById('doc-viewer-azd').contentWindow.initAIE();
			}
			else{
				Columns.next();
				self.setNavButtons();
			}
		}
		else{
			var yOffset = document.getElementById('doc-viewer-azd').contentWindow.config.yOffset;
			var documentHeight = document.getElementById('doc-viewer-azd').contentWindow.config.documentHeight;
			var currentHeight = document.getElementById('doc-viewer-azd').contentWindow.config.currentHeight;
			
			if(documentHeight <= (currentHeight + yOffset) || documentHeight - (currentHeight + yOffset) == 1){
				//Go to next chapter
				if(config.currentChapter < Epub.config.spineList.length - 1 && !config.standAlonePage){
					history.chapterNo = null;
					history.readingDepth = null;
					Toolbar.disableBackButton();
					
					config.currentChapter += 1;
					AME.showLoading();
					self.load();
					
					if(config.azardi_presentation == 'fixed') {
						self.unbindResize();
  						
						if(config.azardi_width && config.azardi_height) {
							if(config.azardi_margins != -1) {
								window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
							} else {
								window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
							}
							window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
						} else if(config.azardi_width) {
							if(config.azardi_margins != -1) {
								window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
							} else {
								window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
							}
							window.innerHeight = BookPref.config.openHeight;
						} else if(config.azardi_height){
							window.innerWidth = BookPref.config.openWidth;
							window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
							
						} else {
							window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
						}
					} else if(config.azardi_presentation == 'fullscreen') {
						self.unbindResize();
						
						width = screen.width || BookPref.config.openWidth;
						height = screen.height || BookPref.config.openHeight;
						window.resizeTo(width, height);
						window.moveTo(0, 0);
					} else if(config.azardi_presentation == null) {
						self.unbindResize();
						self.bindResize();
						window.resizeTo(config.orig_width, config.orig_height);
					}
					
					Justify.getParagraphsDefaultAlignment();
					// self.checkForMathMl();
					// self.checkForSVG();
					//	Hyphenation creates problem if page contains mathml.
					// if(!config.containsMathMl && !config.containsSVG){
						// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
					// }
					BookPref.set();	
					Flow.config.showLastPage = false;
					self.processInternalLinks();
					self.processParagraphs();
					Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//					Notes.highlight();
					// if(!config.containsMathMl && !config.containsSVG){
						// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
					// }
					self.loadAudio();
					self.loadVideo();
					self.loadImages();
				}
				else{
					//	TODO: Disable the next button here.
				}
				document.getElementById('doc-viewer-azd').contentWindow.initAIE();
			}
			else{
				Flow.next();
				self.setNavButtons();
			}
		}
		// document.getElementById('doc-viewer-azd').contentWindow.initAIE();
	}
	
	/**
	 * Function to go to previous viewport/chapter
	 */
	var previous = function(){
		var self = this;
		
		document.getElementById('doc-viewer-azd').contentWindow.initPageVariables();
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			var xOffset = document.getElementById('doc-viewer-azd').contentWindow.config.xOffset;
			
			if(xOffset == 0){
				//Go to previous chapter
				if(config.currentChapter > 0  && !config.standAlonePage){
					history.chapterNo = null;
					history.readingDepth = null;
					Toolbar.disableBackButton();
					
					config.currentChapter -= 1;
					AME.showLoading();
					self.load();
					
					if(config.azardi_presentation == 'fixed') {
						self.unbindResize();
  						
						if(config.azardi_width && config.azardi_height) {
							if(config.azardi_margins != -1) {
								window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
							} else {
								window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
							}
							window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
						} else if(config.azardi_width) {
							if(config.azardi_margins != -1) {
								window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
							} else {
								window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
							}
							window.innerHeight = BookPref.config.openHeight;
						} else if(config.azardi_height){
							window.innerWidth = BookPref.config.openWidth;
							window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
							
						} else {
							window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
						}
					} else if(config.azardi_presentation == 'fullscreen') {
						self.unbindResize();
						
						width = screen.width || BookPref.config.openWidth;
						height = screen.height || BookPref.config.openHeight;
						window.resizeTo(width, height);
						window.moveTo(0, 0);
					} else if(config.azardi_presentation == null) {
						self.unbindResize();
						self.bindResize();
						window.resizeTo(config.orig_width, config.orig_height);
					}
					
					Justify.getParagraphsDefaultAlignment();
					// self.checkForMathMl();
					// self.checkForSVG();
					//	Hyphenation creates problem if page contains mathml.
					// if(!config.containsMathMl && !config.containsSVG){
						// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
					// }
					BookPref.set();
					Columns.config.showLastPage = true;
					Flow.config.showLastPage = true;
					self.processInternalLinks();
					self.processParagraphs();
					Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//					Notes.highlight();
					// if(!config.containsMathMl && !config.containsSVG){
						// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
					// }
					self.loadAudio();
					self.loadVideo();
					self.loadImages();	
				}
				else{
					//	TODO: Disable the previous button here.
				}
				document.getElementById('doc-viewer-azd').contentWindow.initAIE();
			}
			else{
				Columns.previous();
				self.setNavButtons();
			}
		}
		else{
			var yOffset = document.getElementById('doc-viewer-azd').contentWindow.config.yOffset;
			
			if(yOffset == 0){
				//Go to previous chapter
				if(config.currentChapter > 0 && !config.standAlonePage){
					history.chapterNo = null;
					history.readingDepth = null;
					Toolbar.disableBackButton();
					
					config.currentChapter -= 1;
					AME.showLoading();
					self.load();
					
					if(config.azardi_presentation == 'fixed') {
						self.unbindResize();
  						
						if(config.azardi_width && config.azardi_height) {
							if(config.azardi_margins != -1) {
								window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
							} else {
								window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
							}
							window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
						} else if(config.azardi_width) {
							if(config.azardi_margins != -1) {
								window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
							} else {
								window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
							}
							window.innerHeight = BookPref.config.openHeight;
						} else if(config.azardi_height){
							window.innerWidth = BookPref.config.openWidth;
							window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
							
						} else {
							window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
						}
					} else if(config.azardi_presentation == 'fullscreen') {
						self.unbindResize();
						
						width = screen.width || BookPref.config.openWidth;
						height = screen.height || BookPref.config.openHeight;
						window.resizeTo(width, height);
						window.moveTo(0, 0);
					} else if(config.azardi_presentation == null) {
						self.unbindResize();
						self.bindResize();
						window.resizeTo(config.orig_width, config.orig_height);
					}
					
					Justify.getParagraphsDefaultAlignment();
					// self.checkForMathMl();
					// self.checkForSVG();
					//	Hyphenation creates problem if page contains mathml.
					// if(!config.containsMathMl && !config.containsSVG){
						// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
					// }
					BookPref.set();
					Flow.config.showLastPage = true;
					self.processInternalLinks();
					self.processParagraphs();
					Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//					Notes.highlight();
					// if(!config.containsMathMl && !config.containsSVG){
						// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
					// }
					self.loadAudio();
					self.loadVideo();
					self.loadImages();	
				}
				else{
					//	TODO: Disable the previous button here.
				}
				document.getElementById('doc-viewer-azd').contentWindow.initAIE();
			}
			else{
				Flow.previous();
				self.setNavButtons();
			}
		}
	
	}
	
	/**
	 * Function to go to next chapter
	 */
	var nextChapter = function() {
		var self = this;
		
		if(config.currentChapter < Epub.config.spineList.length - 1 && !config.standAlonePage){
			history.chapterNo = null;
			history.readingDepth = null;
			Toolbar.disableBackButton();
			
			config.currentChapter += 1;
			AME.showLoading();
			self.load();
			
			if(config.azardi_presentation == 'fixed') {
				self.unbindResize();
				
				if(config.azardi_width && config.azardi_height) {
					if(config.azardi_margins != -1) {
						window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
					} else {
						window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				} else if(config.azardi_width) {
					if(config.azardi_margins != -1) {
						window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
					} else {
						window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = BookPref.config.openHeight;
				} else if(config.azardi_height){
					window.innerWidth = BookPref.config.openWidth;
					window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
					
				} else {
					window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
				}
			} else if(config.azardi_presentation == 'fullscreen') {
				self.unbindResize();
				
				width = screen.width || BookPref.config.openWidth;
				height = screen.height || BookPref.config.openHeight;
				window.resizeTo(width, height);
				window.moveTo(0, 0);
			} else if(config.azardi_presentation == null) {
				self.unbindResize();
				self.bindResize();
				window.resizeTo(config.orig_width, config.orig_height);
			}
			
			Justify.getParagraphsDefaultAlignment();
			// self.checkForMathMl();
			// self.checkForSVG();
			//	Hyphenation creates problem if page contains mathml.
			// if(!config.containsMathMl && !config.containsSVG){
				// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			// }
			BookPref.set();
			Columns.config.showLastPage = false;
			Flow.config.showLastPage = false;
			self.processInternalLinks();
			self.processParagraphs();
			Notes.getNotes(Library.config.currentBookId, config.currentChapter);
			// Notes.highlight();
			// if(!config.containsMathMl && !config.containsSVG){
				// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			// }
			self.loadAudio();
			self.loadVideo();
			self.loadImages();
			document.getElementById('doc-viewer-azd').contentWindow.initAIE();
		}
	};
	
	/**
	 * Function to go to previous chapter
	 */
	var previousChapter = function() {
		var self = this;
		
		if(config.currentChapter > 0  && !config.standAlonePage){
			history.chapterNo = null;
			history.readingDepth = null;
			Toolbar.disableBackButton();
			
			config.currentChapter -= 1;
			AME.showLoading();
			self.load();
			
			if(config.azardi_presentation == 'fixed') {
				self.unbindResize();
				
				if(config.azardi_width && config.azardi_height) {
					if(config.azardi_margins != -1) {
						window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
					} else {
						window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				} else if(config.azardi_width) {
					if(config.azardi_margins != -1) {
						window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
					} else {
						window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
					}
					window.innerHeight = BookPref.config.openHeight;
				} else if(config.azardi_height){
					window.innerWidth = BookPref.config.openWidth;
					window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
					
				} else {
					window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
				}
			} else if(config.azardi_presentation == 'fullscreen') {
				self.unbindResize();
				
				width = screen.width || BookPref.config.openWidth;
				height = screen.height || BookPref.config.openHeight;
				window.resizeTo(width, height);
				window.moveTo(0, 0);
			} else if(config.azardi_presentation == null) {
				self.unbindResize();
				self.bindResize();
				window.resizeTo(config.orig_width, config.orig_height);
			}
			
			Justify.getParagraphsDefaultAlignment();
			// self.checkForMathMl();
			// self.checkForSVG();
			//	Hyphenation creates problem if page contains mathml.
			// if(!config.containsMathMl && !config.containsSVG){
				// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			// }
			BookPref.set();
			Columns.config.showLastPage = false;
			Flow.config.showLastPage = false;
			self.processInternalLinks();
			self.processParagraphs();
			Notes.getNotes(Library.config.currentBookId, config.currentChapter);
			// Notes.highlight();
			// if(!config.containsMathMl && !config.containsSVG){
				// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
			// }
			self.loadAudio();
			self.loadVideo();
			self.loadImages();
			document.getElementById('doc-viewer-azd').contentWindow.initAIE();
		}
	};
	
	/**
	 * Function to initilize the sequence to go to reading depth
	 */
	var goToReadingDepth = function() {
		var self = this;
		
		if(String(config.currentChapter).indexOf('azd-outofspine-') == -1){
			config.standAlonePage = false;
			self.load();
		} else {
			var fileLocation = config.currentChapter.replace('azd-outofspine-', '');
			config.standAlonePage = true;
			self.loadStandAlonePage(fileLocation);
		}
		
		if(config.azardi_presentation == 'fixed') {
			self.unbindResize();
			
			if(config.azardi_width && config.azardi_height) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
			} else if(config.azardi_width) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = BookPref.config.openHeight;
			} else if(config.azardi_height){
				window.innerWidth = BookPref.config.openWidth;
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				
			} else {
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
		} else if(config.azardi_presentation == 'fullscreen') {
			self.unbindResize();
			
			width = screen.width || BookPref.config.openWidth;
			height = screen.height || BookPref.config.openHeight;
			window.resizeTo(width, height);
			window.moveTo(0, 0);
		} else if(config.azardi_presentation == null) {
			self.unbindResize();
			self.bindResize();
			window.resizeTo(config.orig_width, config.orig_height);
		}
		
		// window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
		// self.load();
		Justify.getParagraphsDefaultAlignment();
		// self.checkForMathMl();
		// self.checkForSVG();
		//	Hyphenation creates problem if page contains mathml.
		// if(!config.containsMathMl && !config.containsSVG){
			// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
		// }
		BookPref.set();
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			Columns.config.showLastPage = false;
			
			Columns.config.goToDepth = config.oldReadingDepth;
				
		}
		else{
			Flow.config.showLastPage = false;
			
			//	No reading depth for non column mode.
			
		}
		
		self.processInternalLinks();
		self.processParagraphs();
		Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//			Notes.highlight();
		// if(!config.containsMathMl && !config.containsSVG){
			// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
		// }
		self.loadAudio();
		self.loadVideo();
		self.loadImages();
		
		document.getElementById('doc-viewer-azd').contentWindow.initAIE();
	}
	
	/**
	 * Function to load the chapter and go to note
	 * @params id Id of the note
	 * @params chapter Chapter no.
	 */
	var goToNote = function(id, chapter){
		var self = this;

		// $('#but_toc').click();
		
		if(chapter.indexOf('azd-outofspine-') == -1){
			config.currentChapter = parseInt(chapter);
			config.standAlonePage = false;
			AME.showLoading();
			self.load();
		} else {
			config.currentChapter = chapter;
			var fileLocation = config.currentChapter.replace('azd-outofspine-', '');
			config.standAlonePage = true;
			AME.showLoading();
			self.loadStandAlonePage(fileLocation);
		}
		
		if(config.azardi_presentation == 'fixed') {
			self.unbindResize();
			
			if(config.azardi_width && config.azardi_height) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
			} else if(config.azardi_width) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = BookPref.config.openHeight;
			} else if(config.azardi_height){
				window.innerWidth = BookPref.config.openWidth;
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				
			} else {
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
		} else if(config.azardi_presentation == 'fullscreen') {
			self.unbindResize();
			
			width = screen.width || BookPref.config.openWidth;
			height = screen.height || BookPref.config.openHeight;
			window.resizeTo(width, height);
			window.moveTo(0, 0);
		} else if(config.azardi_presentation == null) {
			self.unbindResize();
			self.bindResize();
			window.resizeTo(config.orig_width, config.orig_height);
		}
		
		Justify.getParagraphsDefaultAlignment();
		// self.checkForMathMl();
		// self.checkForSVG();
		//	Hyphenation creates problem if page contains mathml.
		// if(!config.containsMathMl && !config.containsSVG){
			// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
		// }
		BookPref.set();
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			Columns.config.showLastPage = false;
			Columns.config.goToNoteId = id;
		}
		else{
			Flow.config.showLastPage = false;
			Flow.config.goToNoteId = id;
		}
		
		self.processInternalLinks();
		self.processParagraphs();
		Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//		Notes.highlight();
		// if(!config.containsMathMl && !config.containsSVG){
			// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
		// }
		self.loadAudio();
		self.loadVideo();
		self.loadImages();
		
		document.getElementById('doc-viewer-azd').contentWindow.initAIE();
	}
	
	/**
	 * Function used to navigate from internal links
	 * @params reerenceLocation Location of the next chapter.
	 */
	var goToFromInternalLinks = function(referenceLocation){
		var self = this;
		
		history.chapterNo = config.currentChapter;
		Toolbar.enableBackButton();
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			history.readingDepth = Columns.config.currentReadingDepth;
		}
		
		var fileLocation = referenceLocation.split('#')[0];
		var goToReference = referenceLocation.split('#')[1];
		var spineId = Epub.getSpineIdFromFileLocation(fileLocation);
		if(typeof spineId == 'undefined') {
			Toolbar.disableBackButton();
			history.chapterNo = null;
			history.readingDepth = null;
			return;
		}
		var nextChapter = Epub.getChapterIdFromSpineId(spineId);
		if(nextChapter !== false){
			config.currentChapter = nextChapter;
			config.standAlonePage = false;
			
			AME.showLoading();
			//Load chapter page
			self.load();
		} else {
			if(Epub.config.previewSpine) {
				AME.showLoading();
				self.loadPreviewPage();
			} else {
				config.standAlonePage = true;
				//Change currentChapter to some custom name, need it when we save chapter information on closing
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				//Load chapter page that is not in spine order
				self.loadStandAlonePage(fileLocation);
			}
		}
		
		if(config.azardi_presentation == 'fixed') {
			self.unbindResize();
			
			if(config.azardi_width && config.azardi_height) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
			} else if(config.azardi_width) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = BookPref.config.openHeight;
			} else if(config.azardi_height){
				window.innerWidth = BookPref.config.openWidth;
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				
			} else {
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
		} else if(config.azardi_presentation == 'fullscreen') {
			self.unbindResize();
			
			width = screen.width || BookPref.config.openWidth;
			height = screen.height || BookPref.config.openHeight;
			window.resizeTo(width, height);
			window.moveTo(0, 0);
		} else if(config.azardi_presentation == null) {
			self.unbindResize();
			self.bindResize();
			window.resizeTo(config.orig_width, config.orig_height);
		}
		
		Justify.getParagraphsDefaultAlignment();
		//Set the book preferences.
		BookPref.set();
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			Columns.config.showLastPage = false;
			if(typeof goToReference != 'undefined'){
				Columns.config.goToReference = goToReference;
			}	
		}
		else{
			Flow.config.showLastPage = false;
			if(typeof goToReference != 'undefined'){
				Flow.config.goToReference = goToReference;
			}
		}
		
		self.processInternalLinks();
		self.processParagraphs();
		Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//		Notes.highlight();
		// if(!config.containsMathMl && !config.containsSVG){
			// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
		// }
		self.loadAudio();
		self.loadVideo();
		self.loadImages();
		
		document.getElementById('doc-viewer-azd').contentWindow.initAIE();
	}
	
	/**
	 * Function used for navigation using page-nav.
	 * @params referenceLocation Destination location
	 */
	var goToFromPageNav = function (referenceLocation) {
		var self = this;
		
		history.chapterNo = config.currentChapter;
		
		Toolbar.enableBackButton();
		
		var fileLocation = referenceLocation.split('#')[0];
		var goToReference = referenceLocation.split('#')[1];
		var spineId = Epub.getSpineIdFromFileLocation(fileLocation);
		var nextChapter = Epub.getChapterIdFromSpineId(spineId);
		if(nextChapter !== false){
			config.currentChapter = nextChapter;
			config.standAlonePage = false;
			
			AME.showLoading();
			self.load();
		} else {
			if(Epub.config.previewSpine) {
				AME.showLoading();
				self.loadPreviewPage();
			} else {
				config.standAlonePage = true;
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				self.loadStandAlonePage(fileLocation);
			}
		}
		
		if(config.azardi_presentation == 'fixed') {
			self.unbindResize();
			
			if(config.azardi_width && config.azardi_height) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
			} else if(config.azardi_width) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = BookPref.config.openHeight;
			} else if(config.azardi_height) {
				window.innerWidth = BookPref.config.openWidth;
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				
			} else {
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
		} else if(config.azardi_presentation == 'fullscreen') {
			self.unbindResize();
			
			width = screen.width || BookPref.config.openWidth;
			height = screen.height || BookPref.config.openHeight;
			window.resizeTo(width, height);
			window.moveTo(0, 0);
		} else if(config.azardi_presentation == null) {
			self.unbindResize();
			self.bindResize();
			window.resizeTo(config.orig_width, config.orig_height);
		}
		
		Justify.getParagraphsDefaultAlignment();
		BookPref.set();
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			Columns.config.showLastPage = false;
			if(typeof goToReference != 'undefined'){
				Columns.config.goToReference = goToReference;
			}	
		}
		else{
			Flow.config.showLastPage = false;
			if(typeof goToReference != 'undefined'){
				Flow.config.goToReference = goToReference;
			}
		}
		self.processInternalLinks();
		self.processParagraphs();
		Notes.getNotes(Library.config.currentBookId, config.currentChapter);
		self.loadAudio();
		self.loadVideo();
		self.loadImages();
		
		document.getElementById('doc-viewer-azd').contentWindow.initAIE();
	};
	
	/**
	 * Function used for navigation using landmark.
	 * @params referenceLocation Destination location
	 */
	var goToFromLandmark = function (referenceLocation) {
		var self = this;
		
		history.chapterNo = config.currentChapter;
		
		Toolbar.enableBackButton();
		
		var fileLocation = referenceLocation.split('#')[0];
		var goToReference = referenceLocation.split('#')[1];
		var spineId = Epub.getSpineIdFromFileLocation(fileLocation);
		var nextChapter = Epub.getChapterIdFromSpineId(spineId);
		if(nextChapter !== false){
			config.currentChapter = nextChapter;
			config.standAlonePage = false;
			
			AME.showLoading();
			self.load();
		} else {
			if(Epub.config.previewSpine) {
				AME.showLoading();
				self.loadPreviewPage();
			} else {
				config.standAlonePage = true;
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				self.loadStandAlonePage(fileLocation);
			}
		}
		
		if(config.azardi_presentation == 'fixed') {
			self.unbindResize();
			
			if(config.azardi_width && config.azardi_height) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
			} else if(config.azardi_width) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = BookPref.config.openHeight;
			} else if(config.azardi_height){
				window.innerWidth = BookPref.config.openWidth;
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				
			} else {
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
		} else if(config.azardi_presentation == 'fullscreen') {
			self.unbindResize();
			
			width = screen.width || BookPref.config.openWidth;
			height = screen.height || BookPref.config.openHeight;
			window.resizeTo(width, height);
			window.moveTo(0, 0);
		} else if(config.azardi_presentation == null) {
			self.unbindResize();
			self.bindResize();
			window.resizeTo(config.orig_width, config.orig_height);
		}
		
		Justify.getParagraphsDefaultAlignment();
		BookPref.set();
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			Columns.config.showLastPage = false;
			if(typeof goToReference != 'undefined'){
				Columns.config.goToReference = goToReference;
			}	
		}
		else{
			Flow.config.showLastPage = false;
			if(typeof goToReference != 'undefined'){
				Flow.config.goToReference = goToReference;
			}
		}
		self.processInternalLinks();
		self.processParagraphs();
		Notes.getNotes(Library.config.currentBookId, config.currentChapter);
		self.loadAudio();
		self.loadVideo();
		self.loadImages();
		
		document.getElementById('doc-viewer-azd').contentWindow.initAIE();
	}
	
	/**
	 * Loads the cover page as current chapter
	 * @params tocType Type of the toc navigating from
	 */
	var goToCoverPage = function(tocType) {
		var self = this;
		
		history.chapterNo = config.currentChapter;
		
		Toolbar.enableBackButton();
		Toolbar.disableColumnsButton();
		Toolbar.disableColumnRuleButton();
		Toolbar.disableFontSizeButton();
		Toolbar.disableResetButton();
		Toolbar.disableMarginSpaceButton();
		Toolbar.disableLineSpaceButton();
		Toolbar.disableJustifyButton();
		// Toolbar.disableColorButton();
		Toolbar.disableHyphenateButton();
		Toolbar.disableBookPrefButton();
		Toolbar.disableAudioOptionsButton();
		
		if(BookPref.config.columns && tocType == 'side'){
			history.readingDepth = Columns.config.currentReadingDepth;
		}
		
		config.standAlonePage = true;
		// config.currentChapter = 'azd-outofspine-'+fileLocation;
		AME.showLoading();
		self.removeCssReference();
		// self.removeJsReference();
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html('');
		
		var snippet = '';
		if(Epub.config.cover_type === 'image') {
			snippet = '<div class="azardi-cover-image" style="background-image:url(\'$background-image$\');"><div class="azardi-cover-details"><p>$title$</p><p>$author$</p></div></div>';
		} else {
			snippet = '<div class="azardi-cover-avatar" style="background-image:url(\'$background-image$\');"><div class="azardi-cover-details"><p>$title$</p><p>$author$</p></div></div>';
		}
		snippet = snippet.replace('$background-image$', Epub.config.cover_image);
		snippet = snippet.replace('$title$', Epub.config.title);
		snippet = snippet.replace('$author$', Epub.config.author);
		
		$('#doc-viewer-azd').contents().find('#custom-style-igp').html(snippet);
		
		if(BookPref.config.columns){
			Columns.init(BookPref.config.noOfCols);
		} else {
			Flow.init();
		}
		
		Fonts.setMin();
		Hyphen.setNone();
		Line.setMin();
		Margin.setMin();
		
		self.setNavButtons();
		
		self.setCoverDim();
		
		AME.hideLoading();
	};
	
	/**
	 * Sets the height of the cover inside cover page
	 */
	var setCoverDim = function() {
		var Dim = document.getElementById('doc-viewer-azd').contentWindow.getInnerDim();
		
		$('#doc-viewer-azd').contents().find('.azardi-cover-image, .azardi-cover-avatar').css({
			// 'width': Dim.width,
			'height': Dim.height
		});
		
		$(window).bind('resize', function(){
			var Dim = document.getElementById('doc-viewer-azd').contentWindow.getInnerDim();
		
			$('#doc-viewer-azd').contents().find('.azardi-cover-image, .azardi-cover-avatar').css({
				// 'width': Dim.width,
				'height': Dim.height
			});
		});
	};
	
	/**
	 * Function used for navigation using toc.
	 * @params referenceLocation Destination location
     * @params tocType Type of toc used to initiate the navigation side|full-page
	 */
	var goToFromToc = function(referenceLocation, tocType){
		var self = this;
		
		history.chapterNo = config.currentChapter;
		
		Toolbar.enableBackButton();
		if(BookPref.config.columns && config.azardi_presentation != 'fixed' && tocType == 'side'){
			history.readingDepth = Columns.config.currentReadingDepth;
		}
		
		var fileLocation = referenceLocation.split('#')[0];
		var goToReference = referenceLocation.split('#')[1];
		var spineId = Epub.getSpineIdFromFileLocation(fileLocation);
		var nextChapter = Epub.getChapterIdFromSpineId(spineId);
		if(nextChapter !== false){
			config.currentChapter = nextChapter;
			config.standAlonePage = false;
			
			AME.showLoading();
			//Load chapter page
			self.load();
		} else {
			if(Epub.config.previewSpine) {
				AME.showLoading();
				self.loadPreviewPage();
			} else {
				config.standAlonePage = true;
				//Change currentChapter to some custom name, need it when we save chapter information on closing
				config.currentChapter = 'azd-outofspine-'+fileLocation;
				AME.showLoading();
				//Load chapter page that is not in spine order
				self.loadStandAlonePage(fileLocation);
			}
		}
		
		if(config.azardi_presentation == 'fixed') {
			self.unbindResize();
			//While resizing the window, we need to consider the margin as well
			//Fixed layout dimensions do not include margins, reader has to add it explicitly.
			if(config.azardi_width && config.azardi_height) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
			} else if(config.azardi_width) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = BookPref.config.openHeight;
			} else if(config.azardi_height){
				window.innerWidth = BookPref.config.openWidth;
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				
			} else {
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
		} else if(config.azardi_presentation == 'fullscreen') {
			self.unbindResize();
			
			width = screen.width || BookPref.config.openWidth;
			height = screen.height || BookPref.config.openHeight;
			window.resizeTo(width, height);
			window.moveTo(0, 0);
		} else if(config.azardi_presentation == null) {
			self.unbindResize();
			self.bindResize();
			window.resizeTo(config.orig_width, config.orig_height);
		}
		
		Justify.getParagraphsDefaultAlignment();
		//Set the book preferences
		BookPref.set();
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			Columns.config.showLastPage = false;
			if(typeof goToReference != 'undefined'){
				Columns.config.goToReference = goToReference;
			}	
		}
		else{
			Flow.config.showLastPage = false;
			if(typeof goToReference != 'undefined'){
				Flow.config.goToReference = goToReference;
			}
		}
		self.processInternalLinks();
		self.processParagraphs();
		Notes.getNotes(Library.config.currentBookId, config.currentChapter);
		self.loadAudio();
		self.loadVideo();
		self.loadImages();
		
		document.getElementById('doc-viewer-azd').contentWindow.initAIE();
	};
	
	var goBack = function(){
		var self = this;
		
		config.currentChapter = history.chapterNo;
		
		if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
			Columns.config.goToDepth = history.readingDepth;
		}
		
		if(String(config.currentChapter).indexOf('azd-outofspine-') == -1){
			config.standAlonePage = false;
			AME.showLoading();
			self.load();
		} else {
			var fileLocation = config.currentChapter.replace('azd-outofspine-', '');
			config.standAlonePage = true;
			AME.showLoading();
			self.loadStandAlonePage(fileLocation);
		}
		
		if(config.azardi_presentation == 'fixed') {
			self.unbindResize();
			
			if(config.azardi_width && config.azardi_height) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
			} else if(config.azardi_width) {
				if(config.azardi_margins != -1) {
					window.innerWidth = parseInt(config.azardi_width) + (2 * config.azardi_margins);
				} else {
					window.innerWidth = parseInt(config.azardi_width) + (2 * BookPref.config.margin);
				}
				window.innerHeight = BookPref.config.openHeight;
			} else if(config.azardi_height){
				window.innerWidth = BookPref.config.openWidth;
				window.innerHeight = parseInt(config.azardi_height)+parseInt(config.toolbar_height);
				
			} else {
				window.resizeTo(BookPref.config.openWidth, BookPref.config.openHeight);
			}
		} else if(config.azardi_presentation == 'fullscreen') {
			self.unbindResize();
			
			width = screen.width || BookPref.config.openWidth;
			height = screen.height || BookPref.config.openHeight;
			window.resizeTo(width, height);
			window.moveTo(0, 0);
		} else if(config.azardi_presentation == null) {
			self.unbindResize();
			self.bindResize();
			window.resizeTo(config.orig_width, config.orig_height);
		}
		
		Justify.getParagraphsDefaultAlignment();
		// self.checkForMathMl();
		// self.checkForSVG();
		//	Hyphenation creates problem if page contains mathml.
		// if(!config.containsMathMl && !config.containsSVG){
			// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
		// }
		BookPref.set();
		
		self.processInternalLinks();
		self.processParagraphs();
		Notes.getNotes(Library.config.currentBookId, config.currentChapter);
//		Notes.highlight();
		// if(!config.containsMathMl && !config.containsSVG){
			// document.getElementById('doc-viewer-azd').contentWindow.Hyphenator.run();
		// }
		self.loadAudio();
		self.loadVideo();
		self.loadImages();
		document.getElementById('doc-viewer-azd').contentWindow.initAIE();
	}
	
	/**
	 * Change the src of audio tag in this chapter to point to audios in buffer location
	 */
	var loadAudio = function(){
		var self = this;
		
		var audioObj, audioUrl, filename;
		
		var $audio = $('#doc-viewer-azd').contents().find('#custom-style-igp audio');
		//	If no audio is found on the page.
		if($audio.length == 0) {
			return false;
		} else {
			$audio.each(function() {
				var audRef = $(this).attr('src');
				if(typeof audRef == 'undefined') {
					//	If audio tag does not have any src attribute then check if it contains source tag.
					var $source = $(this).find('source');
					if($source.length == 0) {
						//	If no source tag then skip this audio.
						return true;
					} else {
						//	If the source tag is found, then check if it contains src.
						var sourceRef = $source.attr('src');
						if(typeof sourceRef == 'undefined') {
							//	If source tag does not have any src attribute then skip this audio.
							return true;
						} else {
							// If the source attribute is not empty then set the new src of this source tag.
							$source.remove();
							filename = sourceRef.split('/').pop();
							audioObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
							if(audioObj.exists()){
								audioUrl = getURL(audioObj);
								// $(this).attr('preload', 'none');
								$(this).attr('src', audioUrl);
							} else {
								Epub.config.containErrors = true;
							}
						}
					}
				} else {
					//	If the source attribute is not empty then set the new src of this audio tag.
					filename = audRef.split('/').pop();
					audioObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
					if(audioObj.exists()) {
						audioUrl = getURL(audioObj);
						// $(this).attr('preload', 'none');
						$(this).attr('src', audioUrl);
					} else {
						Epub.config.containErrors = true;
					}
				}
			});
		}
		
		// var audioObj;
		// var audioUrl;
		// var $audio = $('#doc-viewer-azd').contents().find('#custom-style-igp audio source');
		// if($audio.length == 0){
			// return false;
		// }
		// else{
			// $audio.each(function(){
				// for(var i = 0; i < Epub.config.bookData.length; i++){
					// var filename = $(this).attr('src').split('/').pop();
					// if(unescape(filename) == unescape(Epub.config.bookData[i]['href'].split('/').pop())){
						// audioObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
						// audioUrl = getURL(audioObj);
						// // alert(audioUrl);
						// $(this).parent().attr('preload', 'none')
						// $(this).parent().attr('src', audioUrl);
						// $(this).remove();
						// break;
					// }
				// }
			// });
		// }
	};
	
	/**
	 * Change the src of video tag in this chapter to point to audios in buffer location
	 */
	var loadVideo = function(){
		var self = this;
		
		var videoObj, videoUrl, filename;
		
		var $video = $('#doc-viewer-azd').contents().find('#custom-style-igp video');
		//	If no video is found on the page.
		if($video.length == 0) {
			return false;
		} else {
			$video.each(function() {
				var vidRef = $(this).attr('src');
				if(typeof vidRef == 'undefined') {
					//	If video tag does not have any src attribute then check if it contains source tag.
					var $source = $(this).find('source');
					if($source.length == 0) {
						//	If no source tag then skip this video.
						return true;
					} else {
						//	If the source tag is found, then check if it contains src.
						var sourceRef = $source.attr('src');
						if(typeof sourceRef == 'undefined') {
							//	If source tag does not have any src attribute then skip this video.
							return true;
						} else {
							// If the source attribute is not empty then set the new src of this source tag.
							$source.remove();
							filename = sourceRef.split('/').pop();
							videoObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
							if(videoObj.exists()){
								videoUrl = getURL(videoObj);
								$(this).attr('preload', 'none');
								// $(this).removeAttr('autoplay');
								$(this).attr('src', videoUrl);
							} else {
								Epub.config.containErrors = true;
							}
							
						}
					}
				} else {
					//	If the source attribute is not empty then set the new src of this video tag.
					filename = vidRef.split('/').pop();
					videoObj = getFileObject(['ProfD', 'quizfx', 'buffer', Epub.config.epubId, filename]);
					if(videoObj.exists()) {
						videoUrl = getURL(videoObj);
						$(this).attr('preload', 'none');
						// $(this).removeAttr('autoplay');
						$(this).attr('src', videoUrl);
					} else {
						Epub.config.containErrors = true;
					}
					
				}
			});
		}
	};
	
	/**
	 * This function loads all the images in chapter with the data-uris, and when all the images are loaded then
	 * it calls columns/flow init function
	 */
	var loadImages = function(){
		var self = this;
		
		var $img = $('#doc-viewer-azd').contents().find('#custom-style-igp img');
		var  j = 0;
		var totalImages = $img.length;
		
		if(totalImages == 0){
			Toolbar.enableUserPrefButton();
			if(config.azardi_presentation == 'fixed') {
				Toolbar.disableColumnsButton();
				Toolbar.disableColumnRuleButton();
				Toolbar.disableFontSizeButton();
				Toolbar.disableResetButton();
				Toolbar.disableMarginSpaceButton();
				Toolbar.disableLineSpaceButton();
				Toolbar.disableJustifyButton();
				// Toolbar.disableColorButton();
				Toolbar.disableHyphenateButton();
				Toolbar.disableBookPrefButton();
				Toolbar.disableAudioOptionsButton();
					
				Flow.init();
			} else if(Chapter.config.azardi_margins != -1) {
				Toolbar.disableMarginSpaceButton();
			} else {
				Toolbar.enableColumnsButton();
				Toolbar.enableColumnRuleButton();
				Toolbar.enableFontSizeButton();
				Toolbar.enableResetButton();
				Toolbar.enableMarginSpaceButton();
				Toolbar.enableLineSpaceButton();
				Toolbar.enableJustifyButton();
				// Toolbar.enableColorButton();
				Toolbar.enableHyphenateButton();
				Toolbar.enableBookPrefButton();
				// Toolbar.enableAudioOptionsButton();

				if(BookPref.config.columns){
					Columns.init(BookPref.config.noOfCols);
				} else {
					Flow.init();
				}
			}
			
			self.setNavButtons();
//			Justify.getParagraphsDefaultAlignment();
//			BookPref.set();
			Notes.highlight();
			if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
				if(Columns.config.goToNoteId != ''){
					Columns.goToInternalLinkReference(Columns.config.goToNoteId);
					Columns.config.goToNoteId = '';
				}
			}
			else{
				if(Flow.config.goToNoteId != ''){
					Flow.goToInternalLinkReference(Flow.config.goToNoteId);
					Flow.config.goToNoteId = '';
				}
			}
			
			AME.hideLoading();
			if(Epub.config.containErrors){
				Epub.displayWarning();
				Epub.config.containErrors = false;
			}
		}
		
		$img.each(function(){
			var foundInManifest = false;
			for(var i = 0; i < Epub.config.bookData.length; i++){
				if(unescape($(this).attr('src').split('/').pop()) == unescape(Epub.config.bookData[i]['href'].split('/').pop())){
					foundInManifest = true;
					if(Epub.config.bookData[i]['available'] == 'true'){
						$(this).bind('dblclick', function () {
							AME.openImage($(this).attr('src'));
						});
						$(this).attr('src', Epub.config.bookData[i]['data']).load(function(){
							j = j + 1;
							if(j == totalImages){
								Toolbar.enableUserPrefButton();
								if(config.azardi_presentation == 'fixed') {
									Toolbar.disableColumnsButton();
									Toolbar.disableColumnRuleButton();
									Toolbar.disableFontSizeButton();
									Toolbar.disableResetButton();
									Toolbar.disableMarginSpaceButton();
									Toolbar.disableLineSpaceButton();
									Toolbar.disableJustifyButton();
									// Toolbar.disableColorButton();
									Toolbar.disableHyphenateButton();
									Toolbar.disableBookPrefButton();
									Toolbar.disableAudioOptionsButton();
										
									Flow.init();
								} else if(Chapter.config.azardi_margins != -1) {
									Toolbar.disableMarginSpaceButton();
								} else {
									Toolbar.enableColumnsButton();
									Toolbar.enableColumnRuleButton();
									Toolbar.enableFontSizeButton();
									Toolbar.enableResetButton();
									Toolbar.enableMarginSpaceButton();
									Toolbar.enableLineSpaceButton();
									Toolbar.enableJustifyButton();
									// Toolbar.enableColorButton();
									Toolbar.enableHyphenateButton();
									Toolbar.enableBookPrefButton();
									// Toolbar.enableAudioOptionsButton();
					
									if(BookPref.config.columns){
										Columns.init(BookPref.config.noOfCols);
									} else {
										Flow.init();
									}
								}
								self.setNavButtons();
	//							Justify.getParagraphsDefaultAlignment();
	//							BookPref.set();
								Notes.highlight();
								if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
									if(Columns.config.goToNoteId != ''){
										Columns.goToInternalLinkReference(Columns.config.goToNoteId);
										Columns.config.goToNoteId = '';
									}
								}
								else{
									if(Flow.config.goToNoteId != ''){
										Flow.goToInternalLinkReference(Flow.config.goToNoteId);
										Flow.config.goToNoteId = '';
									}
								}
								
								AME.hideLoading();
								if(Epub.config.containErrors){
									Epub.displayWarning();
									Epub.config.containErrors = false;
								}
							}
						});
					} else {
						j = j + 1;
						if(j == totalImages){
							Toolbar.enableUserPrefButton();
							if(config.azardi_presentation == 'fixed') {
								Toolbar.disableColumnsButton();
								Toolbar.disableColumnRuleButton();
								Toolbar.disableFontSizeButton();
								Toolbar.disableResetButton();
								Toolbar.disableMarginSpaceButton();
								Toolbar.disableLineSpaceButton();
								Toolbar.disableJustifyButton();
								// Toolbar.disableColorButton();
								Toolbar.disableHyphenateButton();
								Toolbar.disableBookPrefButton();
								Toolbar.disableAudioOptionsButton();
									
								Flow.init();
							} else if(Chapter.config.azardi_margins != -1) {
								Toolbar.disableMarginSpaceButton();
							} else {
								Toolbar.enableColumnsButton();
								Toolbar.enableColumnRuleButton();
								Toolbar.enableFontSizeButton();
								Toolbar.enableResetButton();
								Toolbar.enableMarginSpaceButton();
								Toolbar.enableLineSpaceButton();
								Toolbar.enableJustifyButton();
								// Toolbar.enableColorButton();
								Toolbar.enableHyphenateButton();
								Toolbar.enableBookPrefButton();
								// Toolbar.enableAudioOptionsButton();
				
								if(BookPref.config.columns){
									Columns.init(BookPref.config.noOfCols);
								} else {
									Flow.init();
								}
							}
							self.setNavButtons();
//							Justify.getParagraphsDefaultAlignment();
//							BookPref.set();
							Notes.highlight();
							if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
								if(Columns.config.goToNoteId != ''){
									Columns.goToInternalLinkReference(Columns.config.goToNoteId);
									Columns.config.goToNoteId = '';
								}
							}
							else{
								if(Flow.config.goToNoteId != ''){
									Flow.goToInternalLinkReference(Flow.config.goToNoteId);
									Flow.config.goToNoteId = '';
								}
							}
							
							AME.hideLoading();
							if(Epub.config.containErrors){
								Epub.displayWarning();
								Epub.config.containErrors = false;
							}
						}
					}
					break;
				}
			}
			if(!foundInManifest) {
				Epub.config.containErrors = true;
				j = j + 1;
				if(j == totalImages){
					Toolbar.enableUserPrefButton();
					if(config.azardi_presentation == 'fixed') {
						Toolbar.disableColumnsButton();
						Toolbar.disableColumnRuleButton();
						Toolbar.disableFontSizeButton();
						Toolbar.disableResetButton();
						Toolbar.disableMarginSpaceButton();
						Toolbar.disableLineSpaceButton();
						Toolbar.disableJustifyButton();
						// Toolbar.disableColorButton();
						Toolbar.disableHyphenateButton();
						Toolbar.disableBookPrefButton();
						Toolbar.disableAudioOptionsButton();
							
						Flow.init();
					} else if(Chapter.config.azardi_margins != -1) {
						Toolbar.disableMarginSpaceButton();
					} else {
						Toolbar.enableColumnsButton();
						Toolbar.enableColumnRuleButton();
						Toolbar.enableFontSizeButton();
						Toolbar.enableResetButton();
						Toolbar.enableMarginSpaceButton();
						Toolbar.enableLineSpaceButton();
						Toolbar.enableJustifyButton();
						// Toolbar.enableColorButton();
						Toolbar.enableHyphenateButton();
						Toolbar.enableBookPrefButton();
						// Toolbar.enableAudioOptionsButton();
		
						if(BookPref.config.columns){
							Columns.init(BookPref.config.noOfCols);
						} else {
							Flow.init();
						}
					}
					self.setNavButtons();
	//							Justify.getParagraphsDefaultAlignment();
	//							BookPref.set();
					Notes.highlight();
					if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
						if(Columns.config.goToNoteId != ''){
							Columns.goToInternalLinkReference(Columns.config.goToNoteId);
							Columns.config.goToNoteId = '';
						}
					}
					else{
						if(Flow.config.goToNoteId != ''){
							Flow.goToInternalLinkReference(Flow.config.goToNoteId);
							Flow.config.goToNoteId = '';
						}
					}
					
					AME.hideLoading();
					if(Epub.config.containErrors){
						Epub.displayWarning();
						Epub.config.containErrors = false;
					}
				}
			}
		});
	};
	
	// var checkForMathMl = function () {
		// var self = this;
// 		
		// if ($('#doc-viewer-azd').contents().find('math').length > 0){
		  	// config.containsMathMl = true;
		// }
		// else{
			// config.containsMathMl = false;
		// }
	// }
// 	
	// var checkForSVG = function () {
		// var self = this;
// 		
		// if ($('#doc-viewer-azd').contents().find('svg').length > 0){
		  	// config.containsSVG = true;
		// }
		// else{
			// config.containsSVG = false;
		// }
	// }
	
	/**
	 * Set the states of navigation buttons
	 */
	var setNavButtons = function() {
		var self = this;
		
		Toolbar.enablePreviousButton();
		Toolbar.enableNextButton();
		document.getElementById('doc-viewer-azd').contentWindow.initPageVariables();
		
		var currentChapter, spineListLength;
		if(config.standAlonePage) {
			currentChapter = 0;
			spineListLength = 1;
		} else {
			currentChapter = config.currentChapter;
			spineListLength = Epub.config.spineList.length;
		}
		
		if(currentChapter == 0) {
			//	If on the first page.
			if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
				//	If in columns mode.
				var xOffset = document.getElementById('doc-viewer-azd').contentWindow.config.xOffset;
				if(xOffset == 0) {
					Toolbar.disablePreviousButton();
				}
			} else {
				//	If in flow mode.
				var yOffset = document.getElementById('doc-viewer-azd').contentWindow.config.yOffset;
				if(yOffset == 0){
					Toolbar.disablePreviousButton();
				}
			}
		} 
		
		if(currentChapter == spineListLength - 1) {
			//	If on the last page.
			if(BookPref.config.columns && config.azardi_presentation != 'fixed'){
				//	If in columns mode.
				var xOffset = document.getElementById('doc-viewer-azd').contentWindow.config.xOffset;
				var documentWidth = document.getElementById('doc-viewer-azd').contentWindow.config.documentWidth;
				var currentWidth = document.getElementById('doc-viewer-azd').contentWindow.config.currentWidth;
				if(documentWidth == (currentWidth + xOffset)){
					Toolbar.disableNextButton();
				}
			} else {
				//	If in flow mode.
				var yOffset = document.getElementById('doc-viewer-azd').contentWindow.config.yOffset;
				var documentHeight = document.getElementById('doc-viewer-azd').contentWindow.config.documentHeight;
				var currentHeight = document.getElementById('doc-viewer-azd').contentWindow.config.currentHeight;
				
				if(documentHeight <= (currentHeight + yOffset) || documentHeight - (currentHeight + yOffset) == 1){
					Toolbar.disableNextButton();
				}
			}
		}
	};
	
	/**
	 * Bind resize event
	 */
	var bindResize = function() {
		var self = this;
		
		$('#doc-viewer-azd').bind('resize', function() {
			config.orig_width = window.outerWidth;
			config.orig_height = window.outerHeight;
		});
	};
	
	/**
	 * Unbind resize event
	 */
	var unbindResize = function() {
		var self = this;
		
		$('#doc-viewer-azd').unbind('resize');
	};
	
	/**
	 * Initilize the reader's audio player if chapter has smil data associated with it
	 * @params mediaOverlay media overlay property of the current chapter
	 */
	var initAudioPlayer = function(mediaOverlay) {
		if(mediaOverlay != '') {
			Toolbar.enableAudioOptionsButton();
		} else {
			Toolbar.disableAudioOptionsButton();
		}
		Toolbar.hideAudioControls();
		AME.paused_handle();
	};
	
	return {
		config: config,
		history: history,
		init: init,
		load: load,
		loadStandAlonePage: loadStandAlonePage,
		processInternalLinks: processInternalLinks,
		processParagraphs: processParagraphs,
		removeCssReference: removeCssReference,
		removeJsReference: removeJsReference,
		setCssReference: setCssReference,
		next: next,
		previous: previous,
		nextChapter: nextChapter,
		previousChapter: previousChapter,
		goToReadingDepth: goToReadingDepth,
		goToFromToc: goToFromToc,
		goToFromLandmark: goToFromLandmark,
		goToFromInternalLinks: goToFromInternalLinks,
		goBack: goBack,
		goToNote: goToNote,
		loadImages: loadImages,
		loadAudio: loadAudio,
		loadVideo: loadVideo,
		// checkForMathMl: checkForMathMl,
		// checkForSVG: checkForSVG,
		setNavButtons: setNavButtons,
		setJsReference: setJsReference,
		loadImageAsPage: loadImageAsPage,
		loadSwfAsPage: loadSwfAsPage,
		loadPdfAsPage: loadPdfAsPage,
		setChapterPresentation: setChapterPresentation,
		showToc: showToc,
		hideToc: hideToc,
		bindResize: bindResize,
		unbindResize: unbindResize,
		setStixFont: setStixFont,
		goToCoverPage: goToCoverPage,
		setCoverDim: setCoverDim,
		goToFromPageNav: goToFromPageNav,
		initAudioPlayer: initAudioPlayer,
		loadPreviewPage: loadPreviewPage
	}
}();