/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary to handle the smil file
 */
var Smil = function() {
	/**
	 * Initilize the smil for use
	 * @params smilId Manifest Id of the .smil file
	 */
	var init = function(smilId) {
		//Get SMIL Data for current chapter.
		var smilData = Epub.getChapterSmilData(smilId);
		var audioPath = smilData[0];
		//Change audio file extension .ogg.
		audioPath = processAudioPath(audioPath);
		//Create the time-line from the smil data.
		var timeLine = createTimeline(smilData);
		//Set the audio tag in page.
		setAudioTag(audioPath);
		//Set the timeline in page.
		setTimeline(timeLine);
		addClass();
	};
	/**
	 * Change the audio file extension to .ogg
	 * @params audioPath Path of the audio file
	 * @returns {string} New audio path
	 */
	var processAudioPath = function(audioPath) {
		var audPathArr = audioPath.split('.');
		var ext = audPathArr.pop();

		if(ext !== 'ogg') {
			audPathArr.push('ogg');
			audioPath = audPathArr.join('.');
		}

		return audioPath;
	};
	/**
	 * Create and append audio tag with new audio path
	 * @params audioPath New audio path for the current chapter
	 */
	var setAudioTag = function(audioPath) {
		var audTag = $(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('audio')).addClass('audioPlayer').append($(document.getElementById('doc-viewer-azd').contentWindow.document.createElement('source')).attr('type', 'audio/ogg').attr('src', audioPath));
		$('#doc-viewer-azd').contents().find('#custom-style-igp').prepend(audTag);
	};
	/**
	 * Create the tile line which AIE can understand
	 * @params smilData The smil data for the current chapter
	 * @return {string} Tileline
	 */
	var createTimeline = function(smilData) {
		var timeLine = '<pre class="timeline">\n';
		for(var i = 1; i < smilData.length; i++) {
			var instruction = smilData[i].join(" ");
			timeLine += instruction + '\n';
		}
		timeLine += '</pre>';

		return timeLine;
	};
	/**
	 * Set the timeline in the page
	 * @params timeLine timeLine string which AIE understand
	 */
	var setTimeline = function(timeLine) {
		$('#doc-viewer-azd').contents().find('#custom-style-igp').prepend(timeLine);
	};
	/**
	 * Wrap the content of page with the class that AIE understands
	 */
	var addClass = function() {
		$('#doc-viewer-azd').contents().find('#custom-style-igp').wrapInner('<div class="aie-audio-text-rw" />');
	};
	return {
		init : init
	};
}();
